/****************************************************************************************
* File name: Load.c
* Purpose:  Loads Data to Memory 
*
* Description:	This Program Acknowledges the Reception of the Load Command and
*				Executes it in Case of Command Consistency. This is the
*				acknowledgment of the Load Command. Only to Data RAM
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "l_hwmap.h"
#include "spulib.h"

/*
This function handles the load commands
*/
void Load(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 temp_buffer[60];
	u4 address , address1, address2, memid, memtype, memblock, length;
	u4 checksum, i, address_ERAM, address_SMCS, calc_checksum;
	
	/*=============================================================================================*/
 						   	
	/* Read the Address1 */
	address1 = buf[0]&0x0000FFFF;

	/* Read the Memory ID */
	memid= address1>>8;
	
	/* Read the Address2 */
	address2 = buf[1]>>16;

	/* Read the Logical Address */
	address = ((address1&0xFF)<<16)|(address2&0xFFFF);
	address_ERAM=K_DMADDR_BASE_EXTDRAM+address;
	address_SMCS=K_DMADDR_BASE_SMCSDPRAM+address;
	
	/* The Data Length */
	length = buf[1]&0xFFFF;

	/* The Memory Type (Note the Load is only allowed to the Data Memory) */
	memtype = (memid >>4)&0x1;
	
	/* The Memory Block */
	memblock = memid &0xF;
		
	/*The Checksum Received with the Data */
	checksum = buf[2+length]>>16;

	/* The Data stored in temp_buffer. If Length is greater than 51 Words
	   Then, Only 51 Words Data are Maintained. Done to Avoid temp_buffer 
	   Overflow */
    if(length <= K_MAX_LOAD_LENGTH)
	{	
		for (i=0;i<length;i++)
		{
			temp_buffer[i]=buf[2+i];	
		}
		
		/*Calculate the Checksum of the Received Data */
		calc_checksum = Crc(temp_buffer,length*2,32);
	}
	else
	{	
		for (i=0;i < K_MAX_LOAD_LENGTH;i++)
		{
			temp_buffer[i]=buf[2+i];
		}

		/*Calculate the Checksum of the Received Data */
		calc_checksum = Crc(temp_buffer, K_MAX_LOAD_LENGTH*2,32);
	}
	
	switch (memtype) /* Look for the DM Bank to be loaded */
	{
	case 1: /* Case of the Data Memory */
		{
			/* Case of the EEPROM, Command will be Rejected */
			if (memblock==3)
			{
				fifoTransmitMsg.Header = K_NACK_LOAD | K_INVALID_MEM_ID;
				fifoTransmitMsg.Block1.NackParameter = memid;
			}
			else if ((address> K_DM_END_DRAM)||((address> K_DM_END_DPRAM)&&(memblock==4)))
			{			 
				/* Check the Address for DRAM ExtRAM and DPRAM */
				/* Invalid Address*/
				fifoTransmitMsg.Header = K_NACK_LOAD | K_INVALID_ADDRESS;
				fifoTransmitMsg.Block1.NackParameter = ((address1&0xFF)<<16)|(address2&0xFFFF);
			}
			else if (checksum!= calc_checksum) /* Verify the Checksum */
			{
				/* Invalid Checksum of the Data to Load */
				fifoTransmitMsg.Header = K_NACK_LOAD | K_INVALID_CHECKSUM_1;
				fifoTransmitMsg.Block1.NackParameter = checksum;
			}
	 		else if ((memblock==1)||(memblock==2)||(memblock==4))	 // RAM
			{
				/* Calculate the Pysical Address */
				if(memblock==2) 
				{
					address=address_ERAM;
				}
				else if(memblock==4) 
				{
					address=address_SMCS;
				}

#ifndef SIMULATOR
				/* Write the Data in the Memory */
				for(i=0;i<length;i++)
				{
					((u4*)address)[i] = temp_buffer[i];
					buf[i] = ((u4*)address)[i];
				}
				
				/* Calculate the Checksum for the Loaded Data*/
				calc_checksum = 0;
				calc_checksum=Crc(buf,length*2,32);

				/* Check the Correctness of the Load */
				if (checksum !=calc_checksum)
				{
					/* Invalid Checksum of the Loaded Data */
					fifoTransmitMsg.Header = K_NACK_LOAD | K_INVALID_CHECKSUM_2;
					fifoTransmitMsg.Block1.NackParameter = calc_checksum;
				}
				else
#endif
				{ 
					/* Positive Acknowledgment */
					fifoTransmitMsg.Header = K_PACK_LOAD;
				}
			}
			else /* Case of Invalid Memory Block */
			{
				fifoTransmitMsg.Header = K_NACK_LOAD | K_INVALID_MEM_ID;
				fifoTransmitMsg.Block1.NackParameter = memid;
			}
		}break;
	default: /* Case of Invalid Memory Type */
		{
			fifoTransmitMsg.Header = K_NACK_LOAD | K_INVALID_MEM_ID;
			fifoTransmitMsg.Block1.NackParameter = memid;
		}break;
	}
	KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
}

