/***************************************************************************************
* File name: T_S_Red.c
* Purpose:  Eliminates the Temporal and the Spatial Redundancy.
*
* Description:	The Temporal Redundancy Reduction Program is Used to Reduce
*				(Remove) the Correlation of the Data in Processing Buffer. The
*				Spatial Redundancy Reduction Packs Again the Samples in 16 Bits
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#include "stdio.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h" 

/*=================================================================================================*/
/*Functions declared in this file
===================================================================================================*/

void TSReductionSRR(COMPRESSION_MODE mode);
void TSReductionForSpectroRampDiff(COMPRESSION_MODE mode);
void TSReductionDiffSRR(COMPRESSION_MODE mode);
void ExplodeMsbAndLsb(u4 *buffer, u4 nbWords);
void Put2BytesIn1Word(u4 *buffer, u4 nbWords);
void ReorderPhoto(COMPRESSION_MODE mode);
void TSReductionForSpectroLLC(COMPRESSION_MODE mode);
void ReorderSpectro(COMPRESSION_MODE mode);
void Reorder1Spectro(COMPRESSION_MODE mode);
void TSReductionForPhotoLLC(COMPRESSION_MODE mode);
void TSReductionForSpectro(COMPRESSION_MODE mode);

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

struct BUFFER_DESCRIPTION SortedDataBuffer;	//contains information about the reduced data.  This structure is transmitted to the compression functions

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern void Map2Pos32 (int *buf, int sizeInWords);
extern int RampDiff(u4 *buf, int size_w, int rrr, int subs, int pixelsPerFrame);

/*=================================================================================================*/

/*
Temporal-spatial reduction:
- Put2ShortsIn1Word to pack data
*/
void TSReductionSRR(COMPRESSION_MODE mode)
{
	u4 sizeInWords;
	u4* buf = ProcBuff+1;	//pointer to the data buffer
	
	/*================================================================================================*/
	/* Reordering the Data
	===================================================================================================*/
	
	/* Read the sizeInWords of the Received Data */
	sizeInWords = ProcBuff[0]; 
	
	/*Check if the Data size is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==K_CM_SPECTRO_LOSSLESS_COMPRESSION)		
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/
	
	/* Read the sizeInWords and Divide it by Two to Get the New size */
	buf[sizeInWords]=0;
	sizeInWords = (sizeInWords+1)>>1;
	
	/* Convert Data from 32 Bit words to 16 Bit words */
	Put2ShortsIn1Word(buf, SortedDataBuffer.Data ,sizeInWords);
	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_2_SHORTS_PER_WORD;
}


/*
Temporal-spatial reduction:
- rampdiff 
- map to positive
*/
void TSReductionForSpectroRampDiff(COMPRESSION_MODE mode)
{
	u4 sizeInWords, nbReadoutsPerSubRamps;
    int key;

	/* Read the size of the Received Data */
	sizeInWords = ProcBuff[0]; 
	nbReadoutsPerSubRamps = DetectorConstantsSpectro.ReadoutsPerSubRamps;
	
	/*Check if the Data size is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==K_CM_SPECTRO_LOSSLESS_COMPRESSION)		
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
		nbReadoutsPerSubRamps = mode.ReadoutsPerRamps;
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/

    key = RampDiff (ProcBuff+1, sizeInWords, mode.ReadoutsPerRamps, nbReadoutsPerSubRamps, DxsSpectro.NbSelectedPixels);
	Map2Pos32 ((int*)(ProcBuff+1), sizeInWords);
	ProcBuff[sizeInWords+1] = key;
	sizeInWords++;
	ProcBuff[0] = sizeInWords;

	MemCopyDmToDm(ProcBuff+1, SortedDataBuffer.Data, sizeInWords);

	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_1_SHORT_PER_WORD_MAPPED_TO_POS;
}

/*
Temporal-spatial reduction:
- simple difference
- Put2ShortsIn1Word to pack data
*/
void TSReductionDiffSRR(COMPRESSION_MODE mode)
{
	u4 sizeInWords, i;
	u4 *buf = ProcBuff+1;  // pointer to the data buffer

	/*================================================================================================*/
	/* Reordering the Data
	===================================================================================================*/

	/* Read the sizeInWords of the Received Data */
	sizeInWords = ProcBuff[0]; 
	
	/*Check if the Data sizeInWords is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==K_CM_SPECTRO_LOSSLESS_COMPRESSION)		
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
	}

	//compute the difference
	for(i=0; i < (sizeInWords-1); i++)
	{
		buf[i] = buf[i+1] - buf[i];
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/
	
	/* Read the sizeInWords and Divide it by Two to Get the New sizeInWords */
	buf[sizeInWords]=0;
	sizeInWords = (sizeInWords+1)>>1;
	
	/* Convert Data from 32 Bit words to 16 Bit words */
	Put2ShortsIn1Word(buf, SortedDataBuffer.Data, sizeInWords);
	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_2_SHORTS_PER_WORD;
}

/*
This function separates the MSB and LSB of all the elements of a buffer.
The source and destination buffer are identical.
The buffer size is doubled by this function.
The source buffer is assumed to contain only 16bits per word.

nbWords is the number of words in the buffer at input
*/
void ExplodeMsbAndLsb(u4 *buffer, u4 nbWords)
{
	int i, pos1, pos2;

	/* Conversion of Data Alignment*/
	pos1 = nbWords*2;
	pos2 = nbWords;
	nbWords--;

	for (i = nbWords; i >= 0; i--)
	{      
		buffer[--pos1] = buffer[i] & 0xFF;
		buffer[--pos2] = buffer[i] >> 8;
	}
	return;
}

/*
This function takes one byte in two consecutive words and append them in one word.
The source and destination buffer are identical
The buffer size is reduced by a factor of two by this function
The source buffer is assumed to contain only 8bits per word.

nbWords is the number of words in the buffer at output 
*/
void Put2BytesIn1Word(u4 *buffer, u4 nbWords)
{
	u4 i, kk=0;

	/*=============================================================================================*/ 
	/* Conversion of Data Alignment*/
	for (i = 0; i <nbWords; i++,kk+=2)
	{      
		buffer[i] = ((buffer[kk]<<8)&0xFF00)|(buffer[kk+1] & 0xff);
	}
	return;
}

/*
Temporal-spatial reduction for photo:
- reordering of data MSB and LSB
*/
void ReorderPhoto(COMPRESSION_MODE mode)
{
	u4 size0, sizeInWords, i, size1;
	u4 *buf = ProcBuff+1; // pointer to the data buffer
	u4 *buf2;

	/* Read the sizeInWords of the Received Data */
	sizeInWords = ProcBuff[0]; 
	
	/*Check if the Data sizeInWords is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==4)
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
	}
	
	/* Explode the data into MSB (First sizeInWords) and LSB (Second sizeInWords)*/
	ExplodeMsbAndLsb(buf, sizeInWords);
	
	/* Calculate the Difference for the MSB part*/
	size1=sizeInWords-1;
	for(i=0;i<size1;i++)
	{
		buf[i] = buf[i+1] - buf[i];
	}
	
	/* Put every 2 successive MSB in one short integer words*/
	size0=(sizeInWords+1)>>1;
	Put2BytesIn1Word (buf, size0);
	
	/* Set the size and the Pointer to the LSBs (Second size) */
	sizeInWords=size1+1;
	buf2 = buf + sizeInWords;
	
	/* Put every 2 successive bytes (LSB) in one short integer*/
	size1=(sizeInWords+1)>>1;
	Put2BytesIn1Word (buf2, size1);
	
	/* Align the processed LSB right after the processed MSB in memory*/
	for (i=0;i<size1;i++)
	{
		buf[size0+i] = buf2[i];
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/
	
	/* Read the Size and Divide it by Two to Get the New Size */
	sizeInWords=size0+size1;
	buf[sizeInWords]=size0;
	buf[sizeInWords+1]=size1;
	buf[sizeInWords+2]=0;
	sizeInWords =(sizeInWords+3)>>1;
	
	/* Convert Data from 32 Bit words to packed 16 Bit words */
	Put2ShortsIn1Word(buf, SortedDataBuffer.Data ,sizeInWords);
	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_2_SHORTS_PER_WORD;
}

/*
Temporal-Spatial reduction for spectro lossless compression mode
*/
void TSReductionForSpectroLLC(COMPRESSION_MODE mode)
{
	u4 size0,sizeInWords, i,size1;
	u4 *buf = ProcBuff+1; // pointer to the data buffer
	u4 *buf2;

	/* Read the sizeInWords of the Received Data */
	sizeInWords = ProcBuff[0]; 
	
	/*Check if the Data sizeInWords is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==K_CM_SPECTRO_LOSSLESS_COMPRESSION)
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
	}

	/* Explode the data into MSB (First Size) and LSB (Second Size)*/
	ExplodeMsbAndLsb(buf, sizeInWords);
	
	/* Calculate the Difference for the MSB part*/
	size1=sizeInWords-1;
	for(i=0;i<size1;i++)
	{
		buf[i] = buf[i+1] - buf[i];
	}

	/* Put every 2 successive MSB in one short integer words*/
	size0=(sizeInWords+1)>>1;
	Put2BytesIn1Word (buf, size0);
	
	/* Set the size and the Pointer to the LSBs (Second Size) */
	sizeInWords=size1+1;
	buf2 = buf + sizeInWords;
	
	/* Put every 2 successive bytes (LSB) in one short integer*/
	size1=(sizeInWords+1)>>1;
	Put2BytesIn1Word (buf2, size1);
	
	/* Align the processed LSB right after the processed MSB in memory*/
	for (i=0;i<size1;i++)
	{
		buf[size0+i]=buf2[i];
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/
	
	/* Read the Size and Divide it by Two to Get the New Size */
	sizeInWords=size0+size1;
	buf[sizeInWords]=size0;
	buf[sizeInWords+1]=size1;
	buf[sizeInWords+2]=0;

	sizeInWords = (sizeInWords+3)>>1;
	
	/* Convert Data from 32 Bit words to 16 Bit words */
	Put2ShortsIn1Word(buf, SortedDataBuffer.Data, sizeInWords);
	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_2_SHORTS_PER_WORD;
}

/*
Temporal-spatial reduction for spectro:
- reordering of data
*/
void ReorderSpectro(COMPRESSION_MODE mode)
{
	u4 sizeInWords,i,j,l=0, nbSubSlopes;
	u4 *buf = ProcBuff+1; // pointer to the data buffer

	/*================================================================================================*/
	/* Reordering the Data
	===================================================================================================*/

	/* Read the size of the Received Data */
	sizeInWords = ProcBuff[0]; 
	
	/*Check if the Data size is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==K_CM_SPECTRO_LOSSLESS_COMPRESSION)		
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
		nbSubSlopes = mode.ReadoutsPerRamps;
	}
	else if(mode.CompressionMode == K_CM_SPECTRO_4_SEC_RESET) /* In 4-seconds Reset Interval, RRR is set to 512*/
	{
		nbSubSlopes = 512 >> DetectorConstantsSpectro.Log2OfReadoutsPerSubRamps;
	}
	else
	{
		nbSubSlopes = mode.ReadoutsPerRamps >> DetectorConstantsSpectro.Log2OfReadoutsPerSubRamps;
	}
	
	//re-order
	for(i=0;i<nbSubSlopes;i++)
	{
		for(j=0;j<sizeInWords;j+=nbSubSlopes)
		{
			SortedDataBuffer.Data[l++] = buf[j+i];
		}
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/
	
	/* Read the size and Divide it by Two to Get the New size */
	/* The new size after SRR */
	sizeInWords =(sizeInWords+1)>>1;
	
	/* Convert Data from 32 Bit words to 16 Bit words */
	Put2ShortsIn1Word(SortedDataBuffer.Data, SortedDataBuffer.Data ,sizeInWords);
	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_2_SHORTS_PER_WORD;
}

/*
Temporal-spatial reduction for spectro:
- simple difference
- reordering of data
*/
void Reorder1Spectro(COMPRESSION_MODE mode)
{
	u4 sizeInWords,i,j,l=0, nbSubSlopes;
	u4 *buf = ProcBuff+1; // pointer to the data buffer

	/*================================================================================================*/
	/* Reordering the Data
	===================================================================================================*/

	/* Read the size of the Received Data */
	sizeInWords = ProcBuff[0]; 
	
	/*Check if the Data size is in the allowed range for Lossless Compression*/
	if(mode.CompressionMode==K_CM_SPECTRO_LOSSLESS_COMPRESSION)		
	{
		if(sizeInWords > K_MAX_SIZE_FOR_LLC)
		{
			Result.DmcHeaderError = 0xFF;
			sizeInWords = K_MAX_SIZE_FOR_LLC;
		}
		nbSubSlopes = mode.ReadoutsPerRamps;
	}
	else if(mode.CompressionMode == K_CM_SPECTRO_4_SEC_RESET) /* In 4-seconds Reset Interval, RRR is set to 512*/
	{
		nbSubSlopes = 512 >> DetectorConstantsSpectro.Log2OfReadoutsPerSubRamps;
	}
	else
	{
		nbSubSlopes = mode.ReadoutsPerRamps >> DetectorConstantsSpectro.Log2OfReadoutsPerSubRamps;
	}
	
	//make difference
	for(i=0; i< (sizeInWords-1); i++)
	{
		buf[i] = buf[i+1] - buf[i];
	}
	
	//re-order
	for(i=0;i<nbSubSlopes;i++)
	{
		for(j=0;j<sizeInWords;j+=nbSubSlopes) 
		{
			SortedDataBuffer.Data[l++] = buf[j+i];
		}
	}
	
	/*=============================================================================================*/
	/* The Spatial Redundancy Reduction
	==================================================================================================*/
	
	/* Read the size and Divide it by Two to Get the New size */
	/* The new size after SRR */
	sizeInWords = (sizeInWords+1)>>1;
	
	/* Convert Data from 32 Bit words to 16 Bit words */
	Put2ShortsIn1Word(SortedDataBuffer.Data, SortedDataBuffer.Data ,sizeInWords);
	SortedDataBuffer.SizeInWords = sizeInWords;
	SortedDataBuffer.Type = K_DATA_TYPE_2_SHORTS_PER_WORD;
}

/*
Temporal-Spatial reduction for photo lossless compression mode
*/
void TSReductionForPhotoLLC(COMPRESSION_MODE mode)
{
	if (mode.BolcDataType == 1) /* Test Pattern */
	{
		TSReductionDiffSRR(mode); // Diff+SRR
	}
	else
	{
		ReorderPhoto(mode);
	}
}

/*
Temporal-spatial reduction for spectroscopy mode
*/
void TSReductionForSpectro(COMPRESSION_MODE mode)
{
	/* Take the adequate Sorting Function */
	switch(DetectorConstantsSpectro.SortingAlgorithm)
	{
		case 0:
		{
			TSReductionSRR(mode); // SRR
		};break;
		case 1:
		{
			TSReductionDiffSRR(mode); // Diff+SRR
		};break;
		case 2:
		{
			ReorderSpectro(mode); // Reorder+SRR
		};break;
		case 3:
		{
			Reorder1Spectro(mode); // Diff+Reorder+SRR
		};break;
		case 4:
		{
			TSReductionForSpectroLLC(mode);
		}break;
		case 5:
		{
			TSReductionForSpectroRampDiff(mode);
		}break;
		default:
		{
			Result.LlcError = ((DetectorConstantsSpectro.SortingAlgorithm << 8) | 0x4000);
			TSReductionSRR(mode); // SRR
		};break;
	}
}
