/*****************************************************************************************
* File name: BUF_Spec.c
* Purpose: Performs Processing for the Buffer Transmission Mode in Spectroscopy.
*
* Description:	This Program Performs Data Compression in Spectroscopy for the
*				BUF Mode According to the Received Compression Parameter. It 
*				Fills the Memory for 10 seconds with the Data, then Sends 
*				them in Tranparent Mode.
*  
*				BUF is the Buffer Transmission Mode
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

#ifdef SIMULATOR
u4 PramBuffer[0x200000];
#endif

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

#ifndef SIMULATOR
extern void MEM_CopyPMToDM(u4 src, u4 *dest, u4 size);
extern void MEM_CopyDMToPM(u4 *src, u4 dest, u4 size);
#endif

/*================================================================================================*/

void SpectroBufferTransmissionMode(COMPRESSION_MODE mode)
{
	u4 firstCompressedEntityPacketIndex;
    u4 bufferSizeIn48BitWords = ((K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS + K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS+1)*2)/3; 
#ifdef SIMULATOR
    u4 bufferSizeIn32BitWords = K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS + K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS +1; 
#endif	
	u4 dataSizeIn16BitWords = K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS *2; 
	u4 dmcHeaderSize16BitWords = (K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS +1)*2; 
	u4 dmcHeaderSize48BitWords = dmcHeaderSize16BitWords/3; 
	u4 i;

	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/

	Result.NbMaintainedRamps = 0;
	Result.GlitchCounter = 0;
	Result.SaturationFlag=0;
	Result.NbIntegratedRamps = 0;

	/*No Raw Channel Data are Sent in Transparent Mode */
	RawChannels.SizeInWords = 0;

	/*=============================================================================================*/

	/* Store 4 Buffers Data in Memory */
	if(BufferTransmissionModeBufferId < 4)
	{
	  	/* DMC Header Extraction and Copy to PramBuffer */
		ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff, K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS);
#ifndef SIMULATOR
	    MEM_CopyDMToPM(ProcBuff, PramBuffer + (BufferTransmissionModeBufferId*bufferSizeIn48BitWords) , dmcHeaderSize16BitWords);
#else
	    MemCopyDmToDm(ProcBuff, PramBuffer + (BufferTransmissionModeBufferId*bufferSizeIn32BitWords) , K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS+1);
#endif
		/* Science Data Extraction and Copy to PramBuffer */
		ReadInCircularBuffer(K_CBUF_SCIENCE_SPECTRO, ProcBuff, K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS); 
#ifndef SIMULATOR
	    MEM_CopyDMToPM(ProcBuff, PramBuffer + (BufferTransmissionModeBufferId*bufferSizeIn48BitWords) + dmcHeaderSize48BitWords , dataSizeIn16BitWords);
#else
	    MemCopyDmToDm(ProcBuff, PramBuffer + (BufferTransmissionModeBufferId*bufferSizeIn32BitWords) +K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS+1 , K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS);
#endif
		BufferTransmissionModeBufferId++;
	}
	/* Allow to fill the Last Buffer (5th) */
	else if (BufferTransmissionModeBufferId == 4)
	{
		/* Last Data Science Data Extraction */
		ReadInCircularBuffer(K_CBUF_SCIENCE_SPECTRO, ProcBuff+K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS+1, K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS); 

		/* Last DMC Header Extraction */
		ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff+1, K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS);
		
		CompressedDataBuffer.Data = ProcBuff + 1;
		CompressedDataBuffer.SizeInWords = K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS + K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS;

		BuildCompressedEntity(mode);
		firstCompressedEntityPacketIndex = mode.PacketIndex;

		/* Write the Compressed Data in the Output Buffer*/
        for(i=0;i<4;i++)
		{
			if (BufferTransmissionModeEnabled == K_YES) // if the buffer transmission has not been interrupted in the meantime
			{
#ifndef SIMULATOR
				MEM_CopyPMToDM(PramBuffer + (i*bufferSizeIn48BitWords) ,ProcBuff+1, bufferSizeIn48BitWords*3);
#else
				MemCopyDmToDm(PramBuffer + (i*bufferSizeIn32BitWords) ,ProcBuff+1, bufferSizeIn32BitWords);
#endif
				MemCopyDmToDm(ProcBuff+K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS+2 ,ProcBuff+K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS+1, K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS);
				
				mode.PacketIndex = firstCompressedEntityPacketIndex - 4 + i; //the packet index must be re-generated
				BuildCompressedEntity(mode);
			}
			else
				break;
		}
		
		/* Get ready for next buffer transmission mode*/
		BufferTransmissionModeBufferId = 0;
		BufferTransmissionModeEnabled = K_NO;
	}
}
