/********************************************************************************
* File name: csw.c
* Purpose:  Gives the Control to a Specific Module for a Specific Compression
*			parameter			 
*
* Description:	The Compression Software Compress the Science Data According 
*				to the Required Compressed Mode. The Compressed Parameter is
*				Received Via the Supervisor. The Application Software Will 
*				Accordingly Start Appropriate Tasks Sequence.
********************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#include "stdio.h"
#include "time.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"


/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

u4 TransparentMode = K_NO; 
u4 IncludeRawChannels = K_NO;
u4 AveragingParametersSummary = 0;

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern void PhotoLosslessCompressionMode(u4 nbFrames, COMPRESSION_MODE mode);
extern void PhotoTransparentMode(u4 nbFrames, COMPRESSION_MODE mode);
extern void PhotoBufferTransmissionMode(COMPRESSION_MODE mode);
extern void PhotoReductionAndCompression(u4 numavg, u4 nbFrames, COMPRESSION_MODE mode);

extern void SpectroReductionAndCompressionMode(u4 nbFrames, COMPRESSION_MODE mode);
extern void SpectroLosslessCompressionMode(u4 nbFrames, COMPRESSION_MODE mode);
extern void SpectroTransparentMode(u4 nbFrames, COMPRESSION_MODE mode);
extern void SpectroBufferTransmissionMode(COMPRESSION_MODE mode);

/*================================================================================================*/

/*
This is the entry point of the CSW task
*/
void CompressionSoftware()
{
	u4 nbFrames;
	struct FIFO_CSW_MSG msg;
	
	/*=============================================================================================*/
	while (1)
	{
		//wait that the supervisor sends a request for compressing buffers
		KS_FIFOGetW(FIFO_CSW, &msg); 
		
		nbFrames = msg.NbFrames;
		
		//default initalization of compression parameters
		AveragingParametersSummary = 0;
		TransparentMode = K_NO;
		Result.DmcHeaderError = 0;
		
		/* Check the Observing Mode*/
		switch(msg.Mode.DetectorType)
		{
		case K_PHOTO: /* Photometry */
			{
				/* Check the Compression Mode in Photometry*/
				switch(msg.Mode.CompressionMode)
				{
				case K_CM_PHOTO_LOSSLESS_COMPRESSION:
					{
#ifdef SIMULATOR
						printf(" Lossless compression mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_NO;
						PhotoLosslessCompressionMode(nbFrames, msg.Mode);
					};break;
				case K_CM_PHOTO_TRANSPARENT:   
					{
#ifdef SIMULATOR
						printf(" Transparent mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						TransparentMode = K_YES;
						IncludeRawChannels=K_NO;
						PhotoTransparentMode(nbFrames, msg.Mode);
					};break;
				case K_CM_PHOTO_DEFAULT:  
					{
#ifdef SIMULATOR
						printf(" Default mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_YES;
						PhotoReductionAndCompression(4, nbFrames, msg.Mode);
					};break;
				case K_CM_PHOTO_DOUBLE_COMPRESSION:  /* in double compr. mode the averaging is done on 8 or more samples instead of 4 usually*/
					{
#ifdef SIMULATOR
						printf(" Double compression mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_YES;
						PhotoReductionAndCompression(8, nbFrames, msg.Mode);
					};break;
				case K_CM_PHOTO_HALF_COMPRESSION:   /* Half compr. mode is used when BOLC is working at 20Hz, the averaging is done on 2 samples instead of 4 usually*/
					{
#ifdef SIMULATOR
						printf(" Half compression mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_YES;
						PhotoReductionAndCompression(2, nbFrames, msg.Mode);
					};break;
				case K_CM_PHOTO_BUFFER_TRANSMISSION:   /* Case of Buffer Transmission Mode in Photometry */
					{
#ifdef SIMULATOR
						printf(" Buffer Transmission mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						PhotoBufferTransmissionMode(msg.Mode);
					};break;
				case K_CM_PHOTO_COMPRESSED_SENSING:  
					{
#ifdef SIMULATOR
						printf(" CS mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_YES;
						PhotoReductionAndCompression(4, nbFrames, msg.Mode);
					};break;
				default: /* Invalid Compression Mode for Photometry */
					{
#ifdef SIMULATOR
						printf(" Invalid Compression mode for Photometry (0x%x)\n", msg.Mode.CompressionMode);
#endif
						PushDmcHeaderError(K_DMC_INVALID_COMPRESSION_MODE);
						
						//discard the headers and readouts that have been buffered
						CircularBufferRemove(K_CBUF_HEADER, msg.NbFrames * K_PHOTO_HEADER_SIZE);
						if (msg.Mode.DataBlockId == 5)
						{
							CircularBufferRemove(K_CBUF_SCIENCE_RED_PHOT, msg.NbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY);
						}
						else
						{
							CircularBufferRemove(K_CBUF_SCIENCE_BLUE_PHOT1, msg.NbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY);
							CircularBufferRemove(K_CBUF_SCIENCE_BLUE_PHOT2, msg.NbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY);
							CircularBufferRemove(K_CBUF_SCIENCE_BLUE_PHOT3, msg.NbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY);
							CircularBufferRemove(K_CBUF_SCIENCE_BLUE_PHOT4, msg.NbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY);
						}
					}break;
				}
			}break;
		case K_SPECTRO: /* Spectroscopy */
			{
				/* Check the Compression Mode in Spectroscopy*/
				switch(msg.Mode.CompressionMode)
				{
				case K_CM_SPECTRO_DEFAULT: 
					{
#ifdef SIMULATOR
						printf(" Default mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_YES;
						SpectroReductionAndCompressionMode(nbFrames, msg.Mode);
					};break;

				case K_CM_SPECTRO_LOSSLESS_COMPRESSION: 
					{
#ifdef SIMULATOR
						printf(" Lossless compression mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_NO;
						SpectroLosslessCompressionMode(nbFrames, msg.Mode);   
					};break;
					
				case K_CM_SPECTRO_TRANSPARENT:  
					{
#ifdef SIMULATOR
						printf(" Transparent mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						TransparentMode = K_YES;
						IncludeRawChannels=K_NO;
						SpectroTransparentMode(nbFrames, msg.Mode);   
					};break;
				case K_CM_SPECTRO_4_SEC_RESET:  
					{
#ifdef SIMULATOR
						printf(" 4 sec reset interval compression mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						IncludeRawChannels=K_YES;
						SpectroReductionAndCompressionMode(nbFrames, msg.Mode);  /* Perform the Default Compression Mode for the Sub_Ramps */
					};break;
				case K_CM_SPECTRO_BUFFER_TRANSMISSION:   
					{
#ifdef SIMULATOR
						printf(" Buffer Transmission mode (0x%x) \n", msg.Mode.CompressionMode);
#endif
						SpectroBufferTransmissionMode(msg.Mode);	 
					};break;
				default: /* Invalid Compression Mode */
					{
#ifdef SIMULATOR
						printf(" Invalid compression mode for Spectroscopy (0x%x)\n", msg.Mode.CompressionMode);
#endif
						PushDmcHeaderError(K_DMC_INVALID_COMPRESSION_MODE);
						
						//discard the headers and readouts that have been buffered
						CircularBufferRemove(K_CBUF_HEADER, msg.NbFrames * K_SPECTRO_HEADER_SIZE);
						CircularBufferRemove(K_CBUF_SCIENCE_SPECTRO, msg.NbFrames * K_SPECTRO_FRAME_SIZE_IN_WORDS);
						
					};break;
				}
			};break;
		}
	}
}
