/***************************************************************************************
* File name: dmch_cp.c
* Purpose:  Performs the Compression of the DEC/MEC Header.
*
* Description:	This Performs the Compression of the DEC/MEC Header using the 
*				Dynamic and Statical Temporal Redundancy Reduction.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"

/*=================================================================================================*/
/*Functions declared in this file
===================================================================================================*/

void fliphdr (u4 *source, u4 columns, u4 lines, u4 *dest);
void DmcHeaderCompression(u4 nbFrames, u4 headerSize, COMPRESSION_MODE mode);
u4 pseudo32 (u4 *source, u4 size, u4 *dest);
void ReduceRedundancyInDmcHeaders (u4 *source, u4 blocks, u4 lines, u4 *tmp_buf);
void dmch_check_phot(u4 *source, u4 size, u4 chop_thresh);

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

COMPRESSED_DMC_HEADER_BUFFER CompressedDmcHeaderBuffer;
u4 ChopperIsInTheSameChopperPlateau[130];

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern int rzip2 (u4 *source, u4 size, u4 *dest, u4 *swap, u4 range); 
extern u4 RZip32(u4 *source, u4 sizeInBytes, u4 *dest, u4 *swap, u4 range);

/*=================================================================================================*/

/*
This function re-organize the data from DMC header
*/
void fliphdr (u4 *source, u4 columns, u4 lines, u4 *dest) 
{
	u4 i, j;

	for (i=0; i < columns; i++) 
	{
		for (j=0; j < lines; j++)
		{
			dest[i*lines + j] = source[j*columns + i];
		}
	}

    return;
}


/*
This function compresses the DMC header.
The result is stored in CompressedDmcHeaderBuffer;
*/
void DmcHeaderCompression(u4 nbFrames, u4 headerSize, COMPRESSION_MODE mode)
{
	u4 compressedBufferSize;
	u4 dmcHeaderBufferSizeInWords = nbFrames * headerSize;
	/*=============================================================================================*/
	/* Reset the Compressed DMCH Size to 0
	===============================================================================================*/

	CompressedDmcHeaderBuffer.SizeInWords = 0;
  
	/*============================================================================================*/
	
	/* Dump the Buffer to a Proccesing Memory */
	ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff, dmcHeaderBufferSizeInWords);
	
	/* Check for Chopper Positions Information for Double Compression Mode  */
/* removed by RO jan09 for SPR 1044
	if (mode.CompressionMode==K_CM_PHOTO_DOUBLE_COMPRESSION)
	{
		dmch_check_phot (ProcBuff, dmcHeaderBufferSizeInWords, DetectorConstantsPhoto.ThresholdForChopper);
	}
*/	
	/* Redundancy Reduction */
   	ReduceRedundancyInDmcHeaders(ProcBuff, nbFrames, headerSize, DramBuffer);
	
	if (SimulationDataParameters.DmcHeaderCompressionAlgorithm == 0) //rzip2 (new)
	{
		fliphdr (ProcBuff, K_HEADER_SIZE, nbFrames, DramBuffer);
		MemCopyDmToDm(DramBuffer, ProcBuff, nbFrames * K_HEADER_SIZE);

		/* Simple  Zero Repetition Suppression */ 
		compressedBufferSize = pseudo32 (ProcBuff, dmcHeaderBufferSizeInWords, DramBuffer); 
		
		/* Coding the Data using the RZip Algorithm */
		compressedBufferSize *= 4; 
		
		compressedBufferSize = rzip2(DramBuffer, compressedBufferSize, ProcBuff, DramBuffer + 0x20000, 3);
		compressedBufferSize = rzip2(ProcBuff, compressedBufferSize, CompressedDmcHeaderBuffer.Data, DramBuffer + 0x20000, 4);
		CompressedDmcHeaderBuffer.Data[0] |= 0x80000000; // enter key for fm header compression
	}
	else
	{
		/* Simple  Zero Repetition Suppression */ 
		compressedBufferSize = pseudo32 (ProcBuff, dmcHeaderBufferSizeInWords, DramBuffer); 
		
		/* Coding the Data using the RZip Algorithm */
		compressedBufferSize *= 4; 
		
		compressedBufferSize = RZip32(DramBuffer, compressedBufferSize, ProcBuff, DramBuffer +0x20000, 6);
		compressedBufferSize = RZip32(ProcBuff, compressedBufferSize, CompressedDmcHeaderBuffer.Data, DramBuffer +0x20000, 6);
	}

	compressedBufferSize = (compressedBufferSize+3)>>2;
	
	/* Store the Compressed DMC Header Size */
	CompressedDmcHeaderBuffer.SizeInWords = compressedBufferSize;
}

/********************************************************************************************
* Function name: pseudo32
*
* Purpose: Suppress the Zero Symbols
*
* Description:	This Function Reduces the Data by Eliminating the Zero Redundancy
*
* Syntax: u4 pseudo32 (u4 *source, u4 size, u4 *dest)
*
* Input: 'source', 'Size'
*
* Output: dest
*
* Return: size in 4-Byte Words
*
* Files Accessed: None
*
* Creation date & author: 03.04.2002 by Roland Ottensamer
* Version, Update date & author: HPSPU-HLSW-OB-V1R0 04.07.2002 by Ahmed Nabil Belbachir
*
********************************************************************************************/

u4 pseudo32 (u4 *source, u4 size, u4 *dest)
{
	/* Declaration of Local Variables
	==============================================================================================*/
  
	u4 ctr = 0; /* source buffer counter [32 bit words] */
	u4 dctr = 0; /* destination buffer counter [single bits] */
	u4 lastpos = 0;

  	/*============================================================================================*/
	
	/* Store the Original Size in 32 bit Words */
	dest[0] = size;
  	
	/* The Zero Suppression Procedure */
	for (ctr = 0, dctr = 0, lastpos = 0; ctr < size; )
    {
		/* Find Next Non-0  Symbol*/
 		for ( ; (source[ctr] == 0) && (ctr < size); ctr++); 
		if (ctr < size) 
		{
			/* Code the Symbol */
			dest[++dctr] = source[ctr]; 
			
			/* Code the Position as Offset from the Last Position*/
			dest[++dctr] = ctr - lastpos; 
			lastpos = ctr;
			ctr++;
		}      
    }
  return dctr+1; /* in 32 bit words */
}

/********************************************************************************************
* Function name: ReduceRedundancyInDmcHeaders
*
* Purpose: Redundancy Reduction
*
* Description:	This Function Eliminate the Redundancy in the DEC/MEC Header in Dynamic and 
*				Static Form 
*
* Syntax: void ReduceRedundancyInDmcHeaders (u4 *source, u4 blocks, u4 lines, u4 *tmp_buf)
*
* Input:  Source Data
*         Blocks: how many Frames
*		  lines: How many words in a Frame
*         Temporal Buffer
* Output: Result in Source Buffer
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 03.08.2002 by Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V1R0 04.08.2002 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void ReduceRedundancyInDmcHeaders (u4 *source, u4 blocks, u4 lines, u4 *tmp_buf)
{
	u4 i,j,k;

	/*============================================================================================*/
	/* Elimination of the Dynamic Redundancy
	==============================================================================================*/

	for (i = 1; i < blocks; i++)
	{
		k=i-1;
		for (j = 0; j < lines; j++)
		{
			tmp_buf[(i*lines)+j] = source[(k*lines)+j]-source[(i*lines)+j]; 
		}
	}

	/*============================================================================================*/
	/* Elimination of the Static Redundancy
	==============================================================================================*/
	
	for (i=lines;i<(2*lines);i++) source[i]= tmp_buf[i];
	for (i = 2; i < blocks; i++)
	{
		for (j = 0; j < lines; j++)
		{
			source[(i*lines)+j] = source[lines+j]-tmp_buf[(i*lines)+j]; 
		}
	}
}

/********************************************************************************************
* Function name: dmch_check_phot,dmch_check_spec
*
* Purpose: Check DEC/MEC Header
*
* Description:	These Functions are used in Double Compression Mode to check DEC/MEC Header Field,
*               Validity, Chopper Position and Grating Position. They generate Look-up Tables that
*               will be used by the Integration Function (Reduction).
*
* Syntax: void dmch_check_phot(u4 *source, u4 size, u4 chop_thresh)
*		  void dmch_check_spec(u4 *source, u4 size, u4 chop_thresh, u4 grat_thresh)
*
* Input:  Source Data
*         size
*		  chopper threshold
*		  grating threshold
* Output: Result in Look-up Tables
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 01.06.2004 by Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V10R1 01.06.2004 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void dmch_check_phot(u4 *source, u4 size, u4 chop_thresh)
{
	u4 i,ChopInfo_Index=source[6];
	u4 CPR_Diff, chopperIndex = 0;

	/*============================================================================================*/
	/* Check for Chopper Motion
	==============================================================================================*/

	for (i=0;i<size;i+=13)
	{
		if(source[i+10]==1) /* CHECK CRDCCP field (1st readout in chopper)*/
		{
			/* Check Up and Down CPR*/
			if (ChopInfo_Index<source[i+6])
				CPR_Diff = source[i+6]-ChopInfo_Index;
			else
				CPR_Diff = ChopInfo_Index-source[i+6];

			/* Check CPR Difference*/
			if (CPR_Diff<chop_thresh) /* Same Chopper Plateau */
			{
				ChopperIsInTheSameChopperPlateau[chopperIndex++] = K_YES;
			}
			else
			{
				ChopperIsInTheSameChopperPlateau[chopperIndex++]= K_NO; /* Different Chopper Plateau */
				ChopInfo_Index=source[i+6];
			}
		}
	}
}

