/****************************************************************************************
* File name: Dump.c
* Purpose:  Acknowledges the Dump Command
*
* Description:	This Program Acknowledges the Reception of the Dump Command, Reads  
*				Memory data and Sends them to DPU in Case of Command Consistency.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "l_hwmap.h"
#include "spulib.h"

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

#ifndef SIMULATOR
extern void MEM_CopyPMToDM(u4 src, u4 *dest, u4 size);
#endif


/*
This function handles the dump commands from DPU
*/
void Dump(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 address , address1, address2, memid, memtype, memblock, checks=0, length;
	u4 i,j,address_ERAM,address_SMCS, NbBlocks,LastLength,newlength;

	/*============================================================================================*/

	/* Read the Address1 */
	address1 = buf[0]&0x0000FFFF;

	/* The Memory ID */
	memid= address1>>8;

	/* Read the Address2 */
	address2 = buf[1]>>16;

	/* The Logical Address */
	address = ((address1&0xFF)<<16)|(address2&0xFFFF);
	address_ERAM=K_DMADDR_BASE_EXTDRAM+address;
	address_SMCS=K_DMADDR_BASE_SMCSDPRAM+address;
	
	/* The Data Length */
	length = buf[1]&0xFFFF;

	/* The Memory Type (Data Memory or Program Memory) */
	memtype = (memid >>4)&0x1;
	
	/* The Memory Block (RAM, Extended RAM or EEPROM) */
	memblock = memid &0xF;

	switch (memtype)
	{
		case 0: /* Case of the Program Memory */
		{
			/* Case of the EEPROM, Command will be Rejected */
			if (memblock==3)
			{
				fifoTransmitMsg.Header = K_NACK_DUMP | K_INVALID_MEM_ID;
				fifoTransmitMsg.Block1.NackParameter = memid;
	
				KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
			}
			else
			/* Case of the PRAM */
			{
				if(length <= K_MAX_DUMP_LENGTH_IN_PM) /* Check the Length and Dump Data from the Memory */
				{
					newlength = roldiv(((length*6)+3), 4);
					fifoTransmitMsg.Header = K_PACK_DUMP_LAST | address1;
					DumpBuffer[0] = fifoTransmitMsg.Header;

					fifoTransmitMsg.Block1.AddressLength = (address2<<16)|(length);
					DumpBuffer[1] = fifoTransmitMsg.Block1.AddressLength;

#ifndef SIMULATOR
					MEM_CopyPMToDM(address,DumpBuffer+2,length*3);
#endif
					/* Calculate the Checksum */
					checks=Crc(DumpBuffer+2,length*3,32)<<16;
					DumpBuffer[newlength+2]=checks;	
					fifoTransmitMsg.Block1.AddressLength = (address2<<16)|(newlength);
					KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
				}
				else  /* Dump Several Blocks if the Data Will Not Fit to the Maximum PUS Size */
				{
					/* read the number of blocks needed to be transmitted */
					NbBlocks = roldiv(length, K_MAX_DUMP_LENGTH_IN_PM);
					newlength = K_MAX_DUMP_LENGTH_IN_DM;
					for(j=0;j<NbBlocks;j++)
					{
						fifoTransmitMsg.Header = K_PACK_DUMP_INTER | (((address>>16)&0x7)|(address1&0xFF00));
						fifoTransmitMsg.Block1.AddressLength = ((address&0xFFFF)<<16)|K_MAX_DUMP_LENGTH_IN_PM;
						DumpBuffer[0] = fifoTransmitMsg.Header;
						DumpBuffer[1] = fifoTransmitMsg.Block1.AddressLength;


#ifndef SIMULATOR
						/* Dump the Data */
						MEM_CopyPMToDM(address,DumpBuffer+2,K_MAX_DUMP_LENGTH_IN_PM*3);
#endif	
		
						/* Calculate the Checksum */
						checks=Crc(DumpBuffer+2,K_MAX_DUMP_LENGTH_IN_PM*3,32)<<16;
					
						DumpBuffer[newlength+2]=checks;	
						fifoTransmitMsg.Block1.AddressLength = (address2<<16)|(newlength);
						
						/* Send the Response Packet to the DPU */
						KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
						
						/* Wait 10 ms to send Data Packet before sending Next Dump Packet*/
						KS_TaskSleep(10);	      		
						address=address+K_MAX_DUMP_LENGTH_IN_PM;
					}
				
					/* Transmit the last block */
					LastLength= length-(NbBlocks*K_MAX_DUMP_LENGTH_IN_PM);
					newlength = roldiv(((LastLength*6)+3), 4);
					fifoTransmitMsg.Header = K_PACK_DUMP_LAST | (((address>>16)&0x7)|(address1&0xFF00));
					fifoTransmitMsg.Block1.AddressLength = ((address&0xFFFF)<<16)|LastLength;
					DumpBuffer[0] = fifoTransmitMsg.Header;
					DumpBuffer[1] = fifoTransmitMsg.Block1.AddressLength;

					#ifndef SIMULATOR
					
					/* Dump the Data */
					MEM_CopyPMToDM(address,DumpBuffer+2,LastLength*3);

					#endif	
		
					/* Calculate the Checksum */
					if (LastLength!=0) 
					{
						checks=Crc(DumpBuffer+2,LastLength*3,32)<<16;
					}
					
					DumpBuffer[newlength+2]=checks;	
					fifoTransmitMsg.Block1.AddressLength = (address2<<16)|(newlength);
					
					/* Send the Response Packet to the DPU */
					KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
				}
			}
		}break;
		
		case 1: /* Case of the Data Memory */
		{
			/* Case of the EEPROM, Command will be Rejected */
			if (memblock==3)
			{
				fifoTransmitMsg.Header = K_NACK_DUMP | K_INVALID_MEM_ID;
				fifoTransmitMsg.Block1.NackParameter = memid;
	
				KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
			}
			else
			{
				/* Check the Address for DRAM ExtRAM and DPRAM */
				if ((address> K_DM_END_DRAM)||((address> K_DM_END_DPRAM)&&(memblock==4)))			
				{
					/* Invalid Address*/
					fifoTransmitMsg.Header = K_NACK_DUMP | K_INVALID_ADDRESS;
					fifoTransmitMsg.Block1.NackParameter = ((address1&0xFF)<<16)|(address2&0xFFFF);
	
					KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
				}
				else
				{
					/* Calculate the Physical in Case of Extended RAM or DPRAM*/
					if(memblock==2) 
					{
						address=address_ERAM;
					}
					else if(memblock==4) 
					{
						address=address_SMCS;
					}

					if(length<= K_MAX_DUMP_LENGTH_IN_DM) /*Check the Length and Dump Data from the Memory*/
					{
						fifoTransmitMsg.Header = K_PACK_DUMP_LAST | address1;
						fifoTransmitMsg.Block1.AddressLength = (address2<<16)|length;
						DumpBuffer[0] = fifoTransmitMsg.Header;
						DumpBuffer[1] = fifoTransmitMsg.Block1.AddressLength;
	  
						/* Dump the Data */
						for (i=0;i<length;i++)
						{
							DumpBuffer[i+2] = ((u4*)address)[i];
						}
				
						/* Calculate the Checksum */
						checks=Crc(DumpBuffer+2,length*2,32)<<16;
						DumpBuffer[length+2]=checks;	

						/* Send the Response Packet to the DPU */
						KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
					}
					else  /* Dump Several Blocks if the Data Will Not Fit to the Maximum PUS Size*/
					{
						/* Read the Number of Blocks Needed to be Transmitted */
						NbBlocks = roldiv(length, K_MAX_DUMP_LENGTH_IN_DM);
						for(j=0;j<NbBlocks;j++)
						{
							fifoTransmitMsg.Header = K_PACK_DUMP_INTER | (((address>>16)&0x7)|(address1&0xFF00));
							fifoTransmitMsg.Block1.AddressLength = ((address&0xFFFF)<<16)|K_MAX_DUMP_LENGTH_IN_DM;
							DumpBuffer[0] = fifoTransmitMsg.Header;
							DumpBuffer[1] = fifoTransmitMsg.Block1.AddressLength;
	  
							/* Dump the Data */
							for (i=0;i<K_MAX_DUMP_LENGTH_IN_DM;i++)
							{
								DumpBuffer[i+2] = ((u4*)address)[i];
							}
		
							/* Calculate the Checksum */
							checks=Crc(DumpBuffer+2,K_MAX_DUMP_LENGTH_IN_DM*2,32)<<16;
							DumpBuffer[K_MAX_DUMP_LENGTH_IN_DM+2]=checks;	

							/* Send the Response Packet to the DPU */
							KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
						
							/* Wait 10 ms to send Data Packet before Next Dump Packet*/
							KS_TaskSleep(10);	      	
						
							address=address+K_MAX_DUMP_LENGTH_IN_DM;
						}
						/* Transmit the Last Block */
						LastLength= length-(NbBlocks*K_MAX_DUMP_LENGTH_IN_DM);
						fifoTransmitMsg.Header = K_PACK_DUMP_LAST | (((address>>16)&0x7)|(address1&0xFF00));
						fifoTransmitMsg.Block1.AddressLength = ((address&0xFFFF)<<16)|LastLength;
						DumpBuffer[0] = fifoTransmitMsg.Header;
						DumpBuffer[1] = fifoTransmitMsg.Block1.AddressLength;
	 
						/* Dump the Data */
						for (i=0;i<LastLength;i++)
						{
							DumpBuffer[i+2] = ((u4*)address)[i];
						}
	
						/* Calculate the Checksum */
            
						if (LastLength!=0) 
						{
							checks=Crc(DumpBuffer+2,LastLength*2,32)<<16;
						}
						DumpBuffer[LastLength+2]=checks;	
	
						/* Send the Response Packet to the DPU */
						KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
					}
				}
			}
		}break;
	default: /* Case of Invalid Memory Type */
		/* Invalid Memory ID (only the Memory Type is Mentioned Here)*/
		{
			fifoTransmitMsg.Header = K_NACK_DUMP | K_INVALID_MEM_ID;
			fifoTransmitMsg.Block1.NackParameter = memid;

			/* Send the Response Packet to the DPU */
			KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
		}break;
	}
}
