/****************************************************************************************
* File name: Dxs.c
* Purpose:  Selects Data from Detector Table.
*
* Description:	The DXS Program is Used to Filter the Input Data with the 
*				Detector Selection Table. Data from Non-Selected Detectors are
*				Ignored.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#include "stdio.h"
#include "time.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spulib.h"
#include "spuAsw.h"


/*=================================================================================================*/
/*Functions declared in this file
===================================================================================================*/

void DetectorSelection(u4 *src, DXS_TABLE* dxsTable,u4 destSize,int nbFrames);
void DxsBluePhotoAllPixels(u4 *src, u4 srcSize, u4 destSize, u4 sample);
void DxsRedPhotoAllPixels(u4 *src, u4 srcSize, u4 destSize);
void DxsSpectroAllPixels(u4 *src, u4 srcSize, u4 destSize);
void Resort(u4 *src, u4 *dest, u4 Size);
void _Put2ShortsIn1Word(u4 *src, u4 *dest , u4 destSizeInWords);
void _Put2ShortsIn2Words(u4 *src, u4 *dest , u4 srcSizeInWords);
void _Put2ShortsIn2WordsSigned(u4 *src, u4 *dest , u4 srcSizeInWords);

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

RAW_CHANNEL_DATA RawChannels;

#ifdef SIMULATOR
u4 DramBuffer[0x40000];			/* Temporary Buffer in DRAM where Processed Data are Put*/
#else
u4* DramBuffer = (u4*) 0x3F580;	/* Temporary Buffer in DRAM where Processed Data are Put*/
#endif

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern u4 IncludeRawChannels;
extern u4 CurrentReadoutCountIn4SecondsRamp;
extern u4 PixelsPositionInSpectroArray[450];


/*
This function performs the detector selection
*/
void Dxs(u4 srcSizeInWords, DXS_TABLE* dxsTable, u4 nbFrames, u4 circBufferId, u4 sample, COMPRESSION_MODE mode)
{
	u4 rawChannelsDataSizeInWords, i, *src;
	u4 nbSelectedDetectors=0, sizeRCBuff1,sizeRCBuff2;
	u4 nextRawIndexPhoto = 0;
	u4 nextRawIndexSpectro = 0;
     
	/*=============================================================================================*/
	/* Reset the Old Size Value*/
	src = DramBuffer;
	ProcBuff[0] =0;
	nbSelectedDetectors = dxsTable->NbSelectedPixels;

	/* The Size of the Resulted Buffer (in 4-B Words) is the Nb of Selected Detectors x Frames*/
	if (TransparentMode == K_YES)
	{
		ProcBuff[0] = (nbFrames*nbSelectedDetectors)>>1 ;
	}
	else
	{
		ProcBuff[0] = nbFrames*nbSelectedDetectors ;
	}


	if ((dxsTable == &DxsSpectro) || (dxsTable == &DxsPhotoRed)) /* Case of Spectroscopy or Red Bolometer*/
	{
		/* Dump one Buffer of the Science data to Processing Memory */
		ReadInCircularBufferAndPut2ShortsIn2Words(circBufferId, src, srcSizeInWords, FALSE);

		if((mode.DetectorType == K_SPECTRO)&&(nbSelectedDetectors == K_NB_USEFUL_PIXELS_IN_SPECTRO_ARRAY)) /* Spec all selected */
		{
			/* The Detector Selection */
			DxsSpectroAllPixels(src,srcSizeInWords*2,ProcBuff[0]);
		}
		else if((mode.DetectorType == K_PHOTO)&&(nbSelectedDetectors == K_NB_PIXELS_IN_PHOTO_BLOCK)) /* Red Phot all selected */
		{
			/* The Detector Selection */
			DxsRedPhotoAllPixels(src,srcSizeInWords*2,ProcBuff[0]);
		}
		else if (nbSelectedDetectors > 0) /* Other cases where some detectors are selected*/
		{
			DetectorSelection(src, dxsTable, ProcBuff[0], nbFrames);
		}
	}
	else /* Case of Blue Bolometer */
	{
		/* Filter the Data with the Table.
		The Result are Stored in the Source Location*/
		if (nbSelectedDetectors == K_NB_PIXELS_IN_BLUE_BOLOMETERS)
		{
			for(i=0;i<4;i++) 
			{
				/* Dump the 4 Science Data Buffers to Process */
				ReadInCircularBufferAndPut2ShortsIn2Words(circBufferId + i, src+(2*i*srcSizeInWords), srcSizeInWords, FALSE);
			}
			
			/* The Detector Selection */
			DxsBluePhotoAllPixels(src,srcSizeInWords*2,ProcBuff[0],sample);
		}
		else
		{
			for(i=0;i<4;i++) 
			{
				/* Dump the 4 Science Data Buffers to Process */
				ReadInCircularBufferAndPut2ShortsIn2Words(circBufferId + i, ProcBuff+1+(2*i*srcSizeInWords), srcSizeInWords, FALSE);
			}

			/* Resort the Data to Have the Correct Order V11.3 */
			Resort(ProcBuff+1, DramBuffer, srcSizeInWords*2);

			/* The Detector Selection */
			DetectorSelection(DramBuffer, dxsTable, ProcBuff[0],nbFrames);//,sample);
		}
	}

	/* Store Raw Channel Data for Some Detectors if no Transparent mode. 
	   Raw Channel Data are not Transmitted in Transparent Mode */
	if(IncludeRawChannels==K_YES)
	{
		switch(mode.DetectorType)
		{
		case K_SPECTRO: /* Spectroscopy */
			{
				if(RawChannelSelectionSpectro.NbSelectedRawChannels != 0)
				{	
					if(RawChannelSelectionSpectro.IndexOfFirstChannel > nbSelectedDetectors) 
					{
						RawChannelSelectionSpectro.IndexOfFirstChannel = 0;
					}

					nextRawIndexSpectro = RawChannelSelectionSpectro.IndexOfFirstChannel;

					/* Size of RCX Data */
					rawChannelsDataSizeInWords = (RawChannelSelectionSpectro.NbSelectedRawChannels*nbFrames)>>1;

					if (mode.CompressionMode==K_CM_SPECTRO_4_SEC_RESET)
					{
						if(CurrentReadoutCountIn4SecondsRamp == 0x200)  //when we have received the first half of the 4sec ramp, we already prepare the nextRawChannelIndex.  It will be used only the next time (= next ramp)
						{
							nextRawIndexSpectro += RawChannelSelectionSpectro.NbSelectedRawChannels;
						}
					}
					else
					{
						nextRawIndexSpectro += RawChannelSelectionSpectro.NbSelectedRawChannels;
					}

					/* RCX Data. Index begins from 0 to DXSNB */
					if(nextRawIndexSpectro <= nbSelectedDetectors)
					{
						Put2ShortsIn1Word(ProcBuff+1+(RawChannelSelectionSpectro.IndexOfFirstChannel*nbFrames), RawChannels.Data , rawChannelsDataSizeInWords);
						RawChannels.SizeInWords = rawChannelsDataSizeInWords;

						/* Reset the RCX Index while overflowing the Number of Selected Detectors */
						if (nextRawIndexSpectro == nbSelectedDetectors) 
						{
							RawChannelSelectionSpectro.IndexOfFirstChannel=0;
						}
						else 
						{
							RawChannelSelectionSpectro.IndexOfFirstChannel = nextRawIndexSpectro;
						}
					}
					else
					{
					    /* To Ensure the Round Robin in the Raw Channel Index i.e. return to Detector Index 0 */
						sizeRCBuff1= ((nbSelectedDetectors - RawChannelSelectionSpectro.IndexOfFirstChannel) * nbFrames)>>1;
						sizeRCBuff2= rawChannelsDataSizeInWords - sizeRCBuff1;
						Put2ShortsIn1Word(ProcBuff+1+(RawChannelSelectionSpectro.IndexOfFirstChannel * nbFrames), RawChannels.Data , sizeRCBuff1);
						Put2ShortsIn1Word(ProcBuff+1, RawChannels.Data + sizeRCBuff1 , sizeRCBuff2);
   						RawChannels.SizeInWords = rawChannelsDataSizeInWords;
						
						RawChannelSelectionSpectro.IndexOfFirstChannel = nextRawIndexSpectro - nbSelectedDetectors;
					}
				}
			}break;
		case K_PHOTO: /* Photometry */
			{
				if(RawChannelSelectionPhoto.NbSelectedRawChannels != 0)
				{	
					if(RawChannelSelectionPhoto.IndexOfFirstChannel > nbSelectedDetectors) 
					{
						RawChannelSelectionPhoto.IndexOfFirstChannel = 0;
					}
					/* Size of RCX Data */
					rawChannelsDataSizeInWords = (RawChannelSelectionPhoto.NbSelectedRawChannels * nbFrames)>>1;

					nextRawIndexPhoto = RawChannelSelectionPhoto.IndexOfFirstChannel + RawChannelSelectionPhoto.NbSelectedRawChannels;

					/* RCX Data. Index begins from 0 to DXSNB */
					if(nextRawIndexPhoto <= nbSelectedDetectors)
					{
						Put2ShortsIn1Word(ProcBuff+1+(RawChannelSelectionPhoto.IndexOfFirstChannel*nbFrames), RawChannels.Data , rawChannelsDataSizeInWords);
						RawChannels.SizeInWords = rawChannelsDataSizeInWords;

						/* Reset the RCX Index while overflowing the Number of Selected Detectors */
						if (nextRawIndexPhoto == nbSelectedDetectors) 
						{
							RawChannelSelectionPhoto.IndexOfFirstChannel = 0;
						}
						else 
						{
							RawChannelSelectionPhoto.IndexOfFirstChannel = nextRawIndexPhoto;
						}
					} 
					else
					{
						/* To Ensure the Round Robin in the Raw Channel Index i.e. return to Detector Index 0 */
						sizeRCBuff1= ((nbSelectedDetectors-RawChannelSelectionPhoto.IndexOfFirstChannel)*nbFrames)>>1;
						sizeRCBuff2= rawChannelsDataSizeInWords - sizeRCBuff1;
						Put2ShortsIn1Word(ProcBuff+1+(RawChannelSelectionPhoto.IndexOfFirstChannel*nbFrames), RawChannels.Data , sizeRCBuff1);
						Put2ShortsIn1Word(ProcBuff+1, RawChannels.Data + sizeRCBuff1 , sizeRCBuff2);
						RawChannels.SizeInWords = rawChannelsDataSizeInWords;
						
						RawChannelSelectionPhoto.IndexOfFirstChannel = nextRawIndexPhoto - nbSelectedDetectors;
					}
				}
			}break;
		}
	}
	else
	{
		RawChannels.SizeInWords = 0;
	}
}

/********************************************************************************************
* Function name: DetectorSelection
*
* Purpose: Selects Data 
*
* Description:	This Function Filters Data from Non-Selected Detectors Using Tables Stored
*				On-Board.
*
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 28.07.2001 by Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 19.01.2006 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void DetectorSelection(u4 *src, DXS_TABLE* dxsTable, u4 destSize, int nbFrames)
{
	u4 *dest = (u4*) ProcBuff+1; /* Destination Buffer */
	u4 nbPixels = dxsTable->Size;
	u4* table = dxsTable->Table;
	
	/*=============================================================================================*/

	/*  Filter Science Data With Table  */
	SelectDetectors(src, dest, table, nbPixels, nbFrames);

	if (TransparentMode == K_YES) /* Concatenate Every Two Readouts in 32 Bit word*/
	{
		Put2ShortsIn1Word(dest, dest, destSize);	
	}
}


/*
This function copies the selected pixels into the destination buffer.
Note that there is an assembly version of this function
*/
void _SelectDetectors(u4* src, u4* dest, u4* table, u4 nbPixelsPerFrame, u4 nbFrames)
{
	u4 pixel;
	u4 frame;
	u4 i = 0;
	u4 srcSize = nbPixelsPerFrame * nbFrames;

	for(pixel=0; pixel < nbPixelsPerFrame; pixel++) 
	{
   		/* If the pixel is valid, write it */
		if (table[pixel] != 0) 
		{
			for(frame = pixel; frame < srcSize; frame += nbPixelsPerFrame) 
			{
				dest[i++] = src[frame];
			}
		}
	}
}

/********************************************************************************************
* Function name: DxsBluePhotoAllPixels
*
* Purpose: Selects Data in blue photometry
*
* Description:	This Function is used in Blue photometry with all detectors selected 
*               for quick Processing.
*
* Syntax: void dxsgo(unsigned int *data_add, unsigned int size_array, int Pix_Nb, unsigned int Size, int Frame_Nb,unsigned int offset )
*
* Input: Pointer to Data, Number of Pixels, Size of Data and Number of Frames, and Offset Address where to Read Data.
*
* Output: The Results are Re-stored in the *data-add
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 26.07.2005 by Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 26.01.2006 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void DxsBluePhotoAllPixels(u4 *src, u4 srcSize,u4 destSize, u4 sample)
{
	u4 *tmp1 = (u4*) ProcBuff+1; /* Destination Buffer */
	u4 *tmp = (u4*) ProcBuff+1; /* Destination Buffer */

	u4 cursor=0, frame, pixel,i;
	u4 sizearray[4];
	u4 jumpframe= K_NB_PIXELS_IN_PHOTO_BLOCK;
	
	sizearray[0]=srcSize;
	sizearray[1]=srcSize*2;
	sizearray[2]=srcSize*3;
	sizearray[3]=srcSize*4;
	

	/*=============================================================================================*/
    /* Check for decimation*/

	if(sample!=0)
	{
		sample--;
		jumpframe *=4;
		destSize >>=2;
		ProcBuff[0]>>=2;
	}

	 /*  Filter Science Data With Table  */
	for (i=0;i<4;i++)
	{
		for(pixel=0; pixel< K_NB_PIXELS_IN_PHOTO_BLOCK; pixel++) 
		{
			for(frame=pixel+cursor+sample; frame<sizearray[i]; frame+=jumpframe) 
				*(tmp1++) = src[frame];
		}
		cursor +=srcSize;
	}


	if (TransparentMode == K_YES) /* Concatenate Every Two Readouts in 32 Bit word*/
	{
		Put2ShortsIn1Word(tmp, tmp, destSize);	
	}
}

/********************************************************************************************
* Function name: DxsRedPhotoAllPixels
*
* Purpose: Selects Data in red photometry
*
* Description:	This Function is used in red photometry with all detectors selected 
*               for quick Processing.
*
* Syntax: void dxsgo(unsigned int *data_add, unsigned int size_array, int Pix_Nb, unsigned int Size, int Frame_Nb,unsigned int offset )
*
* Input: Pointer to Data, Number of Pixels, Size of Data and Number of Frames, and Offset Address where to Read Data.
*
* Output: The Results are Re-stored in the *data-add
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 26.07.2005 by Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 19.01.2006 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void DxsRedPhotoAllPixels(u4 *src, u4 srcSize,u4 destSize)
{
	u4 *dest1 = (u4*) ProcBuff+1; /* Destination Buffer */
	u4 *dest = (u4*) ProcBuff+1; /* Destination Buffer */
	u4 frame, pixel;

  
	 /*  Filter Science Data With Table  */
	for(pixel=0; pixel < K_NB_PIXELS_IN_PHOTO_BLOCK; pixel++) 
	{
		for(frame=pixel; frame < srcSize; frame += K_NB_PIXELS_IN_PHOTO_BLOCK) 
			*dest1++ = src[frame];
	}

	if (TransparentMode == K_YES) /* Concatenate Every Two Readouts in 32 Bit word*/
	{
		Put2ShortsIn1Word(dest, dest, destSize);	
	}
}

/********************************************************************************************
* Function name: DxsSpectroAllPixels
*
* Purpose: Selects Data in spectroscopy
*
* Description:	This Function is used in spectroscopy with all detectors selected 
*               for quick Processing.
*
* Return: None
*
* Creation date & author: 26.07.2005 by Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V12R0 19.09.2005 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void DxsSpectroAllPixels(u4 *src, u4 srcSize, u4 destSize)
{
	u4 *tmp1 = (u4*) ProcBuff+1; /* Destination Buffer */
	u4 *tmp = (u4*) ProcBuff+1; /* Destination Buffer */

	u4 frame, j;

	/*=============================================================================================*/
  
	 /*  Filter Science Data With Table  */
	for(j=0; j < K_NB_USEFUL_PIXELS_IN_SPECTRO_ARRAY; j++) 
	{
		for(frame = PixelsPositionInSpectroArray[j]; frame<srcSize; frame+= K_NB_PIXELS_IN_SPECTRO_ARRAY) 
		{
			*tmp1++ = src[frame];
		}
	}

	if (TransparentMode == K_YES) /* Concatenate Every Two Readouts in 32 Bit word*/
	{
		Put2ShortsIn1Word(tmp, tmp, destSize);	
	}
}

/********************************************************************************************
* Function name: Resort
*
* Purpose: Resorting the Blue Photometry Data 
*
* Description:	This Function is used to maintain the Blue Bolometers data sorted as they
*                arrived from DEC/MEC, such that the detector selection is correctly performed
*
* Syntax: void Resort(u4 *src, u4 *dest, u4 Size)
*
* Input: Pointer to source and destination and Size
*
* Output: The Results are stored in the *dest
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 23.09.2004 by Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V11R3 23.09.2004 by Ahmed Nabil Belbachir
*
********************************************************************************************/

void Resort(u4 *src, u4 *dest, u4 srcSize)
{
	u4 j, cursor=0,sizearray[4], *srcEnd=src + srcSize;
	
	sizearray[0]=0;
	sizearray[1]=srcSize;
	sizearray[2]=2*srcSize;
	sizearray[3]=3*srcSize;

	/*=============================================================================================*/
	
	while (src < srcEnd)
	{
		for(j=0;j<4;j++)
		{
			MemCopyDmToDm(src+sizearray[j],dest+cursor, K_NB_PIXELS_IN_PHOTO_BLOCK);
			cursor += K_NB_PIXELS_IN_PHOTO_BLOCK;
		}
		src += K_NB_PIXELS_IN_PHOTO_BLOCK;
	}
}

/****************************************************************************************
* Function name: _Put2ShortsIn1Word
* Purpose: The input buffer contains 1 short per word.
*          2 consecutive shorts are stored in one word in 'dest'
*          There is no sign management in this routine
* Note: there is an assembly version of this function
*****************************************************************************************/
void _Put2ShortsIn1Word(u4 *src, u4 *dest , u4 destSizeInWords)
{
	u4 l=0;
	u4 k=0;
	
	for(l=0;l<destSizeInWords;l++)
	{
		dest[l] = (src[k]<<16)|(src[1+k]&0xFFFF);
		k+=2;
	}
}

/****************************************************************************************
* Function name: _Put2ShortsIn2Words
* Purpose: The input buffer contains 2 shorts per word.
*          The 2 shorts are stored in two separate words in 'dest'
*          There is no sign management in this routine
* Note: there is an assembly version of this function
*****************************************************************************************/
void _Put2ShortsIn2Words(u4 *src, u4 *dest , u4 srcSizeInWords)
{
	u4 i=0;
	u4 k=0;

	for(i=0; i < srcSizeInWords;i++)
	{
		dest[k++] = src[i] >> 16;
		dest[k++] = src[i] & 0xFFFF;
	}	
}

/****************************************************************************************
* Function name: _Put2ShortsIn2WordsSigned
* Purpose: The input buffer contains 2 signed shorts per word.
*          The 2 signed shorts are stored in two separate signed words in 'dest'
*          There is no sign management in this routine
* Note: there is an assembly version of this function
*****************************************************************************************/
void _Put2ShortsIn2WordsSigned(u4 *src, u4 *dest , u4 srcSizeInWords)
{
	u4 i=0;
	u4 k=0;

	for(i=0; i < srcSizeInWords;i++)
	{
		dest[k++] = ((src[i]>>16)|((src[i]>>31)&0x1)*0xFFFF8000);
		dest[k++] = ((src[i]&0x7FFF)|(src[i]&0x8000)*0x1FFFF);
	}	
}
