/**************************************************************************************
* File name: Hk.c
* Purpose:  Builds HK Packet.
*
* Description:	This Program is Preparing and Filling the HK Packet for the SPU
*				in Idle Mode and During Compression.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#include "stdio.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"
#include "l_hwmap.h"

/*=================================================================================================*/
/*Functions declared in this file
===================================================================================================*/

void Hk(void);
void PushDmcHeaderError(u4 error);

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

struct SPU_HK_PACKET HkPacket;						//the HK Packet buffer that is used by Hk and DataTx
u4 DmcHeaderErrorsBuffer[16];						//If any error encountered in DMC header, store it here (see K_CBUF_DMC_HEADER_ERRORS)
struct MEMORY_SCRUBBING_PARAMS MemoryScrubbingParams;//a container for all memory scrubbing related variables
u4	CbufFreeSpaceSummary;							//a summary of the buffer usage
int ReductionStatus;
int watchme = 0;

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern u4 CircularBufferQuerySpace(u4 bufferId);
extern u4 CbufSize[K_CBUF_NB_CIRCULAR_BUFFERS];

#ifndef SIMULATOR
extern void MemoryScrubbing(void);
#endif


/*
This is the entry point for the housekeeping task
*/
void Hk(void)
{
	u4 memoryResult = 1;
	u4 temp;
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
    fifoTransmitMsg.Header = 0x00870000;
	
	DeclareCircularBuffer(K_CBUF_DMC_HEADER_ERRORS, DmcHeaderErrorsBuffer, &(DmcHeaderErrorsBuffer[15]));

	Result.CompStatus = K_COMP_STATUS_CSW_STOPPED;

	//initialization of the HkPacket members that will never changed during OBS execution
	HkPacket.Header = 0x00870000;
	HkPacket.VersionId = K_SPU_HLSW_VERSION_ID;
	HkPacket.Spare = K_HLSW_BUILDNR;

	//reset memory scrubbing parameters
	MemoryScrubbingParams.CurrentDmAddress = 0;
	MemoryScrubbingParams.CurrentPmAddress = 0;
	MemoryScrubbingParams.indexNextSfDmCorruptedAddress = 0;
	MemoryScrubbingParams.indexNextSfPmCorruptedAddress = 0;
	MemoryScrubbingParams.indexNextDfDmCorruptedAddress = 0;
	MemoryScrubbingParams.indexNextDfPmCorruptedAddress = 0;
	MemoryScrubbingParams.SingleFailureCounter = 0;
	MemoryScrubbingParams.DoubleFailureCounter = 0;


	while(1)
	{
		/* The HK are sent every 1.5 seconds when no data are being processed and
		at the end of every compressed entity when data is being processed*/

		if (KS_SemaTestWT(SEMA_HK, 1900) == RC_OK)	//a compressed entitiy has been processed, send the result to DPU 
		{
			/* Fill the HK Results for Started Compression */
			HkPacket.ObsId = Result.ObsId; 
			HkPacket.PacketIndex = Result.PacketIndex;
			ReductionStatus &= 0xff;
			// TODO: whatever we need in this parameter...
			HkPacket.ReductionAlgo = (Result.ReductionAlgo & 0xff) | (ReductionStatus << 8);
			HkPacket.SaturationFlag = Result.SaturationFlag;
			HkPacket.GlitchCounter = Result.GlitchCounter;
			HkPacket.NbMaintainedRamps = Result.NbMaintainedRamps;
			HkPacket.NbIntegratedRamps = Result.NbIntegratedRamps; 
			HkPacket.RawChannelIndex = Result.RawChannelIndex;
			HkPacket.DmcHeaderError = Result.DmcHeaderError;
			Result.DmcHeaderError = K_DMC_NO_ERROR;
			HkPacket.LlcError = Result.LlcError;
		}
		else	//no data has been processed during the last 1500ms, send normal HK
		{
			/* Fill the HK Packet */
			HkPacket.ObsId = 0; 
			HkPacket.PacketIndex = (Result.PacketIndex)|(Result.CompStatus);
			HkPacket.ReductionAlgo = 0xFF;
			HkPacket.SaturationFlag = 0;
			HkPacket.GlitchCounter = 0;
			HkPacket.NbMaintainedRamps = 0;
			HkPacket.NbIntegratedRamps = 0;
			HkPacket.RawChannelIndex = 0;
			HkPacket.LlcError = 0;
			
			//get DMC header errors if any
			if (CircularBufferQueryUsed(K_CBUF_DMC_HEADER_ERRORS) > 0)
			{
				ReadInCircularBuffer(K_CBUF_DMC_HEADER_ERRORS, &(HkPacket.DmcHeaderError), 1);
			}
			else
			{
				HkPacket.DmcHeaderError = K_DMC_NO_ERROR;
			}
		}				
		
		HkPacket.CounterInc++;
		HkPacket.MemErrorCounter = memoryResult;
#ifndef SIMULATOR
		HkPacket.DmcLinkStatus = DmcLinkConnectionStatus;
		HkPacket.CpuWorkload = KS_WorkloadRead(); 

		/* Scrub the Memory */
		MemoryScrubbing();
		
		/* Check for Memory Errors */
		//Memory_Error_Check();
		memoryResult=(MemoryScrubbingParams.SingleFailureCounter&0xFF)+((MemoryScrubbingParams.DoubleFailureCounter<<8)&0xFF00);
#endif

		//check how buffers are filled:
		temp = roldiv((CircularBufferQuerySpace(K_CBUF_HEADER) * 15), CbufSize[K_CBUF_HEADER]);
		CbufFreeSpaceSummary = temp;
		temp = roldiv((CircularBufferQuerySpace(K_CBUF_SCIENCE_BLUE_PHOT1) * 15), CbufSize[K_CBUF_SCIENCE_BLUE_PHOT1]);
		CbufFreeSpaceSummary |= temp << 4;
		temp = roldiv((CircularBufferQuerySpace(K_CBUF_SCIENCE_RED_PHOT) * 15), CbufSize[K_CBUF_SCIENCE_RED_PHOT]);
		CbufFreeSpaceSummary |= temp << 8;
		temp = roldiv((CircularBufferQuerySpace(K_CBUF_TM) * 15), CbufSize[K_CBUF_TM]);
		CbufFreeSpaceSummary |= temp << 12;

		// monitor the programmable HK parameter
		if ( (SimulationDataParameters.ProgrammableHkAddress <= K_DMADDR_LAST_DRAM) 
				|| ((SimulationDataParameters.ProgrammableHkAddress >= K_DMADDR_BASE_EXTDRAM) 
					&& (SimulationDataParameters.ProgrammableHkAddress <= K_DMADDR_LAST_EXTDRAM)))
			HkPacket.ParameterWished = ((u4*)SimulationDataParameters.ProgrammableHkAddress)[0];
		else
			HkPacket.ParameterWished = 0; 
		
		//HkPacket.ParameterWished = watchme;

		/* Send the HK Packet to the DPU */
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 
	}
}


void PushDmcHeaderError(u4 error)
{
	WriteInCircularBuffer(K_CBUF_DMC_HEADER_ERRORS, &error, 1);
}