/***************************************************************************************
* File name: spu_io.c
* Version.Revision: HPSPU-HLSW-OB-V9R0
*
* Purpose:  Handles the SMCS IO.
*
* Description:	This is the Main Program of the SPU HLSW. It Starts the Connections
*				to DPU and DEC/MEC, Controls the Data Flow to/From the SPU and 
*				Handles the Connection Errors.
*				It Also Starts/Aborts the SPU HLSW Tasks such as the Communication
*				Tasks, the Compression Software Tasks, the HK Task, and the Tasks
*				For the SPU Test Mode.
*
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V9R0 10.05.2004 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header File 
===================================================================================================*/

#include "spu_io.h"

/*=================================================================================================*/
/* External functions 
===================================================================================================*/

extern void SmcsIsr(void);  /*Interrupt Routine in Assembler Code */
extern void dmpsc_isr(void);	/* ISR for DMPSC */
extern void pmpsc_isr(void);	/* ISR for PMPSC */
extern struct MEMORY_SCRUBBING_PARAMS MemoryScrubbingParams;

/*=================================================================================================*/
/* Private Functions 
===================================================================================================*/

void Memory_Error_Check(void);
void MemoryScrubbing(void);
void LinkSmcsRead(int channel);
void Link1355Write(int channel, u4 size, u4 *data);
void ConnectLinkAsSlave(int link);
int ConnectLinkAsMaster(int link);
void ResetLink(int link);

/*=================================================================================================*/
/* Global Variables 
===================================================================================================*/

enum
{
	K_DMC_LINK_NOT_CONNECTED		= 0x00,
	K_DMC_LINK_CONNECTED			= 0xFF,
	K_DMC_LINK_WAITING_CONNECTION	= 0xAA,
};

u4 DmcLinkConnectionStatus = K_DMC_LINK_NOT_CONNECTED;
u4 LinkConnectionTimeCounter = 0;
u4 LinkConnectionTimeOut;
u4 SpuIsSlaveToDmc = K_YES;
int SmcsEvent = SMCS_EVENT;

int ChannelReceiveAddress[3] = 
{
	K_DMADDR_DPUCOMMAND, K_DMADDR_DMCDATA, K_DMADDR_L3DATA
};
int ChannelSendAddress[3] = 
{
	K_DMADDR_ACKTODPU, K_DMADDR_ACKTODMC, K_DMADDR_ACKLINK3
};
int ChannelReceiveSema[3] = 
{
	SPU_RX_FROM_DPU_SEMA, SPU_RX_FROM_DMC_SEMA, SPU_RX_FROM_LINK3_SEMA
};
int ChannelSendSema[3] = 
{
	SPU_TX_TO_DPU_SEMA, SPU_TX_TO_DMC_SEMA, SPU_TX_TO_LINK3_SEMA
};


/*=================================================================================================*/
/* The Main Program
==================================================================================================*/

void DpuConnect(void)
{
	/* Declaration of Local Variables
	===============================================================================================*/

	int firstTime = K_YES;
	int DpuLinkIsConnected = K_NO;

	/*===========================================================================================*/
    /* Set the workload period for debugging purpose */
	KS_WorkloadSetPeriod(TICKFREQ);

	/*===========================================================================================*/
	/* Enable EDAC Check
	=============================================================================================*/
	
	EDAC_EnaDisEDAC(K_HWUNITID_PACS_SPU,K_MEMAREAID_PROGRAM_RAM,K_ENABLE);
	EDAC_EnaDisEDAC(K_HWUNITID_PACS_SPU,K_MEMAREAID_EEPROM,K_ENABLE);
	EDAC_EnaDisEDAC(K_HWUNITID_PACS_SPU,K_MEMAREAID_DATA_RAM,K_ENABLE);
	EDAC_EnaDisEDAC(K_HWUNITID_PACS_SPU,K_MEMAREAID_EXT_DATA_RAM,K_ENABLE);

	KS_IRQSetHandler(K_BPOS_DSPINT_PMPSC, pmpsc_isr); 
	KS_IRQSetHandler(K_BPOS_DSPINT_DMPSC, dmpsc_isr); 

	KS_ISREnable(K_BPOS_DSPINT_PMPSC);
	KS_ISREnable(K_BPOS_DSPINT_DMPSC);



	while(1)
	{
  		/* Set the Link Connection Timeout to 9 seconds
		=============================================================================================*/

		LinkConnectionTimeOut = K_DPU_LINK_CONNECTION_TIMEOUT;

		/*===========================================================================================*/
		/* Reset SMCS chip 
		=============================================================================================*/
    	
		SMCS_ResetSMCSChip();
	
		/*===========================================================================================*/
  		/* Set the DEC/MEC Link Connection Status to Not Connected
		=============================================================================================*/

		DmcLinkConnectionStatus = K_DMC_LINK_NOT_CONNECTED;

		/*===========================================================================================*/
		/* Nominal Configuration of the SMCS
		=============================================================================================*/

		SMCS_ConfigureSMCSNominal();

		/*===========================================================================================*/
		/* Set DSP Interrupt at IRQ2 to Signal SMCS Events
		=============================================================================================*/
	
		KS_IRQSetHandler(K_BPOS_DSPINT_SMCS, SmcsIsr);
		KS_ISREnable(K_BPOS_DSPINT_SMCS);
		SMCS_ClearSMCSIPR();
	
		/*===========================================================================================*/
		/* Configure SMCS Interrupt
		=============================================================================================*/

		SMCS_MaskUnmaskSMCSSubInt(K_BMASK_SMCSINT_CH1_PAR_DIS_ERR | K_BMASK_SMCSINT_CH1_DATA_TXED | K_BMASK_SMCSINT_CH1_EOP_RXED | K_BMASK_SMCSINT_CH2_PAR_DIS_ERR | K_BMASK_SMCSINT_CH2_DATA_TXED | K_BMASK_SMCSINT_CH2_EOP_RXED, K_UNMASK);
	
		/*In Case of using the Third Link, you have to Unmask these Interrupts */

//		SMCS_MaskUnmaskSMCSSubInt(K_BMASK_SMCSINT_CH3_PAR_DIS_ERR | K_BMASK_SMCSINT_CH3_DATA_TXED | K_BMASK_SMCSINT_CH3_DATA_RXED | K_BMASK_SMCSINT_CH3_EOP_RXED, K_UNMASK);
	
		/*===========================================================================================*/
		/* Start HL SMCS Driver
		=============================================================================================*/
	
		if (firstTime==K_YES)
			KS_TaskStart(SMCSHANDLERC);

		/*===========================================================================================*/
		/* Clean the Sema of the Warm Reset 
		=============================================================================================*/
	
		KS_SemaReset(WARM_RESET_SEMA);

		/*===========================================================================================*/
		/* Start the link to DPU as Master
		=============================================================================================*/
	
		DpuLinkIsConnected = ConnectLinkAsMaster(1);				/* DPU Link Start Task */

		/*===========================================================================================*/
		/* Start the SPU Tasks if DPU Link Connection is ON
		=============================================================================================*/

		if (DpuLinkIsConnected==K_YES)
		{
			KS_TaskGroupStart(SPU_INIT);	/* Communication Tasks and HK */
			KS_TaskGroupStart(SPU_ASW);		/* Compression Tasks */
			KS_TaskGroupStart(SPU_TEST_TASKS);	/* Tasks for Test Mode */
		}

		/*===========================================================================================*/
		/* Listen to the Warm Reset or DEC/MEC Link Disconnect Error 
		=============================================================================================*/

		KS_SemaTestW(WARM_RESET_SEMA);

		/*===========================================================================================*/
		/* Abort the SPU Tasks 
		=============================================================================================*/
		
		if(DmcLinkConnectionStatus==K_DMC_LINK_WAITING_CONNECTION) 
			KS_TaskAbort(DMCLINK);			/* Abort DEC/MEC Link_Start Task if it is currently trying to connect but has not succeeded yet*/
		
		/* Abort The SPU Tasks if Connection to DPU or DEC/MEC is Lost */
		if(DpuLinkIsConnected==K_YES)
		{
			KS_TaskGroupAbort(SPU_TEST_TASKS);	/* Tasks for Test Mode */
			KS_TaskGroupAbort(SPU_ASW);			/* Compression SW Tasks */
			KS_TaskGroupAbort(SPU_INIT);		/* Communication Tasks and HK*/
			DpuLinkIsConnected=K_NO;			/* Set The DPU Link Flag to Off "SMCS Chip Reset"*/
		}

		/*===========================================================================================*/
		/* Disable IRQ2 Interrupt 
		=============================================================================================*/

		KS_ISRDisable(K_BPOS_DSPINT_SMCS);


		/*===========================================================================================*/
		/* Disable EDAC Check
		=============================================================================================*/
		
		EDAC_EnaDisEDAC(K_HWUNITID_PACS_SPU,K_MEMAREAID_PROGRAM_RAM,K_DISABLE);
		EDAC_EnaDisEDAC(K_HWUNITID_PACS_SPU,K_MEMAREAID_DATA_RAM,K_DISABLE);

		/*===========================================================================================*/
		/* Not Restart the already started Tasks
		=============================================================================================*/

		firstTime = K_NO;

		/*===========================================================================================*/
		/* Wait Till Last HK Packet is Sent 
		=============================================================================================*/
		
		KS_TaskSleep(2000); 

		/*===========================================================================================*/
	}
}
/* End of the Main Program 
=================================================================================================*/

/*************************************************************************************** 
* Function name: MemoryScrubbing 
*                Memory_Error_Check
* 
* Purpose: Check for Bit Error in Memory 
* 
* Description:  These Function are Used to Scrub the Memory and Correct Single Bit Error in Memory and to Signal 
*				Memory-Word Double Bit Error 
* 
* Syntax: void Memory_Scrub(void)
*		  void Memory_Error_Check(void)
* 
* Input: None 
* 
* Output: None
* 
* Return: None 
* 
* Files Accessed:  CRISA Drivers Operating the EDAC
* 
* Creation date & author: 25.02.2004 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V9R0 10.05.2004 by Ahmed Nabil Belbachir
****************************************************************************************/ 

/* Memory Scrubbing Function
==================================================================================================*/

void MemoryScrubbing(void)
{
#ifndef SIMULATOR
	int i=0;
	const unsigned int nbChecks = 32; //nb of addresses to test each time this function is called

	// read DM
	for (i = MemoryScrubbingParams.CurrentDmAddress; i < (MemoryScrubbingParams.CurrentDmAddress + nbChecks); i++)
	{
		MEM_ReadDMWord(i);
	}
	MemoryScrubbingParams.CurrentDmAddress += nbChecks;
	if (MemoryScrubbingParams.CurrentDmAddress >= 0x0007FFFF) //once the DRAM has been scrubbed, go to Extended DRAM
	{
		if (MemoryScrubbingParams.CurrentDmAddress < 0x40200000) //go to Extended DRAM
		{
			MemoryScrubbingParams.CurrentPmAddress = 0x40200000;
		}
		else if (MemoryScrubbingParams.CurrentPmAddress > 0x4027FFFF) //Extended DRAM has been scrubbed, go back to start address
		{
			MemoryScrubbingParams.CurrentDmAddress = 0;
		}
	}

	// read PM and EEPROM
	for (i = MemoryScrubbingParams.CurrentPmAddress; i < (MemoryScrubbingParams.CurrentPmAddress + nbChecks); i++)
	{
		MEM_ReadPM32bitMSWord(i); //this actually reads the 48bits but only 32 are returned by the function.  This is enough to trigger the EDAC
	}
	MemoryScrubbingParams.CurrentPmAddress += nbChecks;
	if (MemoryScrubbingParams.CurrentPmAddress >= 0x0007FFFF)  //once all PRAM has been scrubbed, go to EEPROM
	{
		if (MemoryScrubbingParams.CurrentPmAddress < 0x000E0000)  //go to EEPROM
		{
			MemoryScrubbingParams.CurrentPmAddress = 0x000E0000;
		}
		else if (MemoryScrubbingParams.CurrentPmAddress > 0x0011FFFF) //EEPROM has been scrubbed, go back to start address
		{
			MemoryScrubbingParams.CurrentPmAddress = 0;
		}
	}
#endif //SIMULATOR
}
/* End of the Memory Scrubbing Function 
=================================================================================================*/
/* Memory Error Check Function
==================================================================================================*/

void Memory_Error_Check(void)
{
#if 0
	/* Declaration of Local Variables
	===============================================================================================*/
	/*EDAC Check Variables */
	u4 PMPSC_IPR=0;
	u4 PMPSCSEFAddress=0;
	u4 PMPSCDEFAddress=0;
	u4 PRAMOrEEPROMFlagForSEF=K_MEMID_PROGRAM_MEMORY;
	u4 PRAMOrEEPROMFlagForDEF=K_MEMID_PROGRAM_MEMORY;
	u4 SubroutineSRForSEF = K_YES;	/* Status returned from called subroutine */
	u4 SubroutineSRForDEF = K_YES;	/* Status returned from called subroutine */
	u4 DMPSC_IPR=0, DMPSCSEFAddress=0, DMPSCDEFAddress=0; 


	/*=============================================================================================*/
	/* Check for EDAC SEF or DEF
	===============================================================================================*/

	/* Program Memory
	===============================================================================================*/

	/* Get PMPSC Interrupt pending register and clear register */
	PSC_GetPMPSCIPR(&PMPSC_IPR);

	/* PMPSC/SEF interrupt */
	if (MEM_CheckBitInDMWord((u4)(&PMPSC_IPR),K_BPOS_PSCINT_SF)== K_NO) 
	{
		SubroutineSRForSEF = EDAC_GetSEFAddressInPM(K_HWUNITID_PACS_SPU, 
							&PRAMOrEEPROMFlagForSEF, &PMPSCSEFAddress);
		if(SubroutineSRForSEF == K_YES) 
		{
			if(PRAMOrEEPROMFlagForSEF==K_MEMAREAID_EEPROM) 
			{
				/* Area is EEPROM */
				EDAC_CorrectSEFInEEPROM(PMPSCSEFAddress);
			}
			if (PRAMOrEEPROMFlagForSEF==K_MEMAREAID_PROGRAM_RAM) 
			{ 
				/* Area is PRAM */
				EDAC_CorrectSEFInPRAM(PMPSCSEFAddress);
			}
		}
		MEM_WriteDMWord(REPORT_BADADDR,PMPSCSEFAddress);
				
		/*Store the Failed Address*/
		WriteInCircularBuffer(K_CBUF_MEM_SCRUB_PM_SF, (u4*)(REPORT_BADADDR), 1);
					
		/*Signal PM SEF*/
		Memory_SEF++;
	}

	/* PMPSC/DEF interrupt */
	if (MEM_CheckBitInDMWord((u4)(&PMPSC_IPR),K_BPOS_PSCINT_DF)== K_NO) 
	{
		SubroutineSRForDEF = EDAC_GetDEFAddressInPM(K_HWUNITID_PACS_SPU,
							&PRAMOrEEPROMFlagForDEF,&PMPSCDEFAddress);

		MEM_WriteDMWord(REPORT_BADADDR,PMPSCDEFAddress);	   

		/*Store the Failed Address*/
		//Memory_Store((u4*)(REPORT_BADADDR), 1,3);
		WriteInCircularBuffer(K_CBUF_MEM_SCRUB_PM_DF, (u4*)(REPORT_BADADDR), 1);
					
		/*Signal PM DEF*/
		Memory_DEF++;
	}
	/*=============================================================================================*/
	/* Data Memory
	===============================================================================================*/

	/* Get DMPSC IPR and clear register */
	PSC_GetDMPSCIPR(&DMPSC_IPR);
			
	/* DMPSC/SEF interrupt */
	if (MEM_CheckBitInDMWord((u4)(&DMPSC_IPR),K_BPOS_PSCINT_SF)== K_NO) 
	{
		EDAC_GetSEFAddressInDM(&DMPSCSEFAddress);
		EDAC_CorrectSEFInDRAM(DMPSCSEFAddress);
		MEM_WriteDMWord(REPORT_BADADDR,DMPSCSEFAddress);
							
		/*Store the Failed Address*/
		WriteInCircularBuffer(K_CBUF_MEM_SCRUB_DM_SF, (u4*)(REPORT_BADADDR), 1);
		
		/*Signal DM SEF*/
		Memory_SEF++;
	}
	/* DMPSC/DEF interrupt */
	if (MEM_CheckBitInDMWord((u4)(&DMPSC_IPR),K_BPOS_PSCINT_DF)== K_NO)
	{
		EDAC_GetDEFAddressInDM(&DMPSCDEFAddress);
		MEM_WriteDMWord(REPORT_BADADDR,DMPSCDEFAddress);

		/*Store the Failed Address*/
		WriteInCircularBuffer(K_CBUF_MEM_SCRUB_DM_SF, (u4*)(REPORT_BADADDR), 1);
					
		/*Signal DM DEF*/
		Memory_DEF++;
	}
	/*===========================================================================================*/
#endif //0
}
/* End of the Memory Error Check Function 
=================================================================================================*/

/*************************************************************************************** 
* Function name: ConnectLinkAsSlave 
* 
* Purpose: Start the Connection to a Remote Sub-unit 
* 
* Description:  This Function is Used to Start a 1355 Link Connection as Slave
*				to a Remote Sub-unit. It Starts the Channel as Slave. 
* 
* Syntax: void ConnectLinkAsSlave(int link)
* 
* Input: Link : 1, 2 or 3 
* 
* Output: None
* 
* Return: None 
* 
* Files Accessed: unsigned int SMCS_StartChannelAsSlave(int link) 
* 
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 07.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

/* The Connect Link Function
==================================================================================================*/

void ConnectLinkAsSlave(int link)
{
	/*  Reset the Counter Before to Start the Link
	=============================================================================================*/

	LinkConnectionTimeCounter = 0;
	
	/*===========================================================================================*/
	/* Connect the Link as Slave 
	=============================================================================================*/

	while(LinkConnectionTimeCounter < LinkConnectionTimeOut) /* Timeout till Link Connection is Established*/
	{
		if (SMCS_StartChannelAsSlave(link) != K_SR_OK)
		{
			LinkConnectionTimeCounter += 50;
			KS_TaskSleep(50); /* A Delay of 50 ms to Synchronize Links */
		}
		else
		{
			// Link is Started. Reset the Counter to Start the New Link
			LinkConnectionTimeCounter = 0;
			break;
		}
	}
	
	/*=============================================================================================*/
	/* When DEC/MEC Link is Started then Set Link Status to ON
	=============================================================================================*/
		
	if(link==2)
		DmcLinkConnectionStatus = K_DMC_LINK_CONNECTED;
	
	/*=============================================================================================*/
	/* Reset the SMC Chip in case the DPU Link Connection is not Established after 9 seconds  
	===============================================================================================*/
		
	if((LinkConnectionTimeCounter!=0) && (link==1))
		KS_SemaSignal(WARM_RESET_SEMA);

	/*===========================================================================================*/
}
/* End of the Connect Link Function 
=================================================================================================*/

/*************************************************************************************** 
* Function name: ConnectLinkAsMaster 
* 
* Purpose: Start the Connection to a Remote Sub-unit 
* 
* Description:  This Function is Used to Start a 1355 Link Connection as Master
*				to a Remote Sub-unit. 
* 
* Syntax: int ConnectLinkAsMaster(int link)
* 
* Input: Link : 1, 2 or 3 
* 
* Output: None
* 
* Return: K_YES if the connection was successful, K_NO otherwise 
* 
* Files Accessed: unsigned int SMCS_StartChannelAsMaster(int link) 
* 
* Creation date & author: 30.04.2003 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R2 07.05.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

int ConnectLinkAsMaster(int link)
{
	/*  Reset the Counter Before to Start the Link
	=============================================================================================*/

	LinkConnectionTimeCounter = 0;
	
	/*===========================================================================================*/
	/* Connect the Link as Master 
	=============================================================================================*/

	while (LinkConnectionTimeCounter < LinkConnectionTimeOut) /* Timeout till Link Connection is Established*/
	{
		if (SMCS_StartChannelAsMaster(link) != K_SR_OK)
		{
			LinkConnectionTimeCounter += 30;
			KS_TaskSleep(30); /* A Delay of 30 ms to Synchronize Links */
		}
		else
		{
			// Link is Started. Reset the Counter to Start the New Link
			LinkConnectionTimeCounter=0;
			break;
		}
	}
	
	/*=============================================================================================*/
	/* When DEC/MEC Link is Started then Set Link Status to ON
	=============================================================================================*/
		
	if(link==2)
		DmcLinkConnectionStatus = K_DMC_LINK_CONNECTED;
	
	/*=============================================================================================*/
	/* Reset the SMC Chip in case the DPU Link Connection is not Established after 9 seconds  
	===============================================================================================*/
		
	if((LinkConnectionTimeCounter!=0) && (link==1))
		KS_SemaSignal(WARM_RESET_SEMA);

	if (LinkConnectionTimeCounter == 0) //the connection was successful
		return K_YES;
	else
		return K_NO;

}

/**************************************************************************************** 
* Function name: ResetLink 
* 
* Purpose: Reset a Link 
* 
* Description:  This Function is Used to Reset the TX and RX section of a Link. 
* 
* Syntax: void ResetLink(int link)
* 
* Input: Link : 1, 2 or 3 
* 
* Output: None
* 
* Return: None 
* 
* Files Accessed: SMCS_ResetChannelTxSection(int link)
*				  SMCS_ResetChannelRxSection(int link) 
* 
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 07.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

void ResetLink(int link)
{
	SMCS_ResetChannelTxSection(link);
	SMCS_ResetChannelRxSection(link);
}

/****************************************************************************************
* Function name: LinkSmcsRead 
* 
* Purpose: Listen to Packets from a Channel
* 
* Description:  This Function is Used to Read Data Packets from a Link. 
* 
* Syntax: void LinkSmcsRead(int Channel)
* 
* Input: Channel : 1, 2 or 3 
* 
* Output: None
* 
* Return: None 
* 
* Files Accessed: SMCS_ReceiveBuffer(Channel, Address, Length)
* 
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 07.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

/* The Link Read Function
==================================================================================================*/
 
void LinkSmcsRead(int channel)
{
	KS_SemaReset(ChannelReceiveSema[channel-1]);
	SMCS_ReceiveBuffer(channel, ChannelReceiveAddress[channel-1], K_1355_MAX_PACKET_LENGTH);
	KS_SemaTestW(ChannelReceiveSema[channel-1]);
}
/* End of the Link Read Function 
=================================================================================================*/

/****************************************************************************************
* Function name: Link1355Write 
* 
* Purpose: Put Data Packets to a Channel
* 
* Description:  This Function is Used to Send Data Packets via A Channel to the Remote
*				Sub-Unit. 
* 
* Syntax: void Link1355Write(int Channel)
* 
* Input: Channel : 1, 2 or 3 
* 
* Output: None
* 
* Return: None 
* 
* Files Accessed: SMCS_TransmitBuffer(Channel, Address, Length)
*				  MemCopyDmToDm(Source, Dest, Length)
* 
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 07.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

#ifndef SIMULATOR
void Link1355Write(int channel, u4 size, u4 *data)
{
	KS_SemaReset(ChannelSendSema[channel-1]);
	MemCopyDmToDm(data,(u4*)ChannelSendAddress[channel-1], K_1355_MAX_PACKET_LENGTH);
	SMCS_TransmitBuffer(K_SMCSCHANNEL_1, K_EOP1, ChannelSendAddress[channel-1], size);
	KS_SemaTestW(ChannelSendSema[channel-1]);
}
#endif

/*************************************************************************************** 
* Function name: DMCLink_Connect 
* 
* Purpose: Start the Connection to DEC/MEC Link 
* 
* Description:  This Function is Used to Start a 1355 Link Connection as Slave
*				to the DEC/MEC 
* 
* Syntax: void DMCLink_Connect(void)
* 
* Input: None 
* 
* Output: None
* 
* Return: None 
* 
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 10.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

void DmcLinkConnect(void)
{
	/*=============================================================================================*/
	/* Set DEC/MEC Link Connection Status to IN-PROGRESS
	=============================================================================================*/

	DmcLinkConnectionStatus = K_DMC_LINK_WAITING_CONNECTION;

	/*===========================================================================================*/
	/* Connect the Link as Master or Slave (SPU-DEC/MEC Case) 
	=============================================================================================*/

	if (SpuIsSlaveToDmc==K_NO) /* Master */
		ConnectLinkAsMaster(2);
	else					/* Slave */
		ConnectLinkAsSlave(2);

	/*=============================================================================================*/
	/* Check in Connection SPU-DEC/MEC  Link is Established
	=============================================================================================*/

	if (DmcLinkConnectionStatus==K_DMC_LINK_CONNECTED)
	{
		/*=============================================================================================*/
		/* Force SPU HLSW to Read DEC/MEC Link
		=============================================================================================*/

		SMCS_WriteSMCSReg(K_DMADD_SMCS_CH2_RX_SAR,K_DMADDR_DMCDATA);
		SMCS_WriteSMCSReg(K_DMADD_SMCS_CH2_RX_EAR,K_DMADDR_L3DATA-1);
	}

	/*=============================================================================================*/
	/* Abort the DEC/MEC Link_Start Task
	=============================================================================================*/
	
	KS_TaskAbort(DMCLINK);				
}

/*************************************************************************************** 
* Function name: SMCSHandlerC 
* 
* Purpose: SMCS Sub-Interrupt Handling 
* 
* Description:  This Function is Used to Handle The Sub-Interrupts Raised by SMCS. Three 
*				Sub-Interrupts are acknowledged: EOP_RXED, DATA_TXED and PAR_DIS_ERR. It is 
*				the C Task with Highest Priority. 
* 
* Syntax: void SMCSHandlerC(void)
* 
* Input: None
* 
* Output: None
* 
* Return: None 
* 
* Files Accessed: SMCS_GetSMCSIPR(unsigned int *SmcsIprLoc);
* 
* Creation date & author: 21.06.2002 by  Ahmed Nabil Belbachir
*
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 07.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/ 

void SMCSHandlerC(void)
{
	/* Declaration of Local Variables
	===============================================================================================*/

	int SmcsIprLoc;
    
	KS_EventEnable(SMCS_EVENT);

   /* Waits the signal from SPU_SMCS_IRQ
   =================================================================================================*/

	while (KS_EventTestW(SMCS_EVENT) == RC_OK)
   {
		/*=============================================================================================*/
		/* Read the SMCS ISR
		===============================================================================================*/

		SMCS_GetSMCSIPR(&SmcsIprLoc);
	
		/*=============================================================================================*/
		/* Signal Reception of EOP from Links 1, 2 and 3
		===============================================================================================*/

		if ((SmcsIprLoc & K_BMASK_SMCSINT_CH1_EOP_RXED) != 0)
		{
			KS_SemaSignal(SPU_RX_FROM_DPU_SEMA);	
		}
		if ((SmcsIprLoc & K_BMASK_SMCSINT_CH2_EOP_RXED) != 0)
		{
			KS_SemaSignal(SPU_RX_FROM_DMC_SEMA);
		}
		/*=============================================================================================*/
		/* Signal Transmission Completed  via Links 1, 2 and 3
		===============================================================================================*/

		if ((SmcsIprLoc & K_BMASK_SMCSINT_CH1_DATA_TXED) != 0)
		{
			KS_SemaSignal(SPU_TX_TO_DPU_SEMA);
	
		}
		if ((SmcsIprLoc & K_BMASK_SMCSINT_CH2_DATA_TXED) != 0)
		{
			KS_SemaSignal(SPU_TX_TO_DMC_SEMA);
		}
		/*=============================================================================================*/
		/* Signal Disconnect Errors on Links 1, 2 and 3
		===============================================================================================*/

		if ((SmcsIprLoc & K_BMASK_SMCSINT_CH1_PAR_DIS_ERR) != 0)
		{
			KS_SemaSignal(WARM_RESET_SEMA);
		}
		if ((SmcsIprLoc & K_BMASK_SMCSINT_CH2_PAR_DIS_ERR) != 0)
		{
	  		/* Set the DEC/MEC Link Connection Status to Not Connected*/
			DmcLinkConnectionStatus = K_DMC_LINK_NOT_CONNECTED;
		}
		/*=============================================================================================*/
	}
}
